/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client;

import org.appspy.admin.client.reports.ReportsMenuTreePanel;
import org.appspy.admin.client.rpc.AdminModuleService;
import org.appspy.admin.client.rpc.AdminModuleServiceAsync;
import org.appspy.admin.client.scheduling.ScheduleMenuTreePanel;
import org.appspy.admin.client.settings.SettingsMenuTreePanel;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.TabPanel;
import com.gwtext.client.widgets.Viewport;
import com.gwtext.client.widgets.layout.AccordionLayout;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class AdminModule implements EntryPoint {

	protected TabPanel mTabPanel = null;
	protected AdminModuleServiceAsync mAdminModuleService = null;

	public void onModuleLoad() {

        final Panel mainPanel = new Panel();
        mainPanel.setLayout(new BorderLayout());


        //setup the west regions layout properties
        BorderLayoutData westLayoutData = new BorderLayoutData(RegionPosition.WEST);
        westLayoutData.setMargins(new Margins(5, 5, 0, 5));
        westLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        westLayoutData.setMinSize(155);
        westLayoutData.setMaxSize(350);
        westLayoutData.setSplit(true);

        //create the west panel and add it to the main panel applying the west region layout properties
		Panel menuPanel = new Panel();
		menuPanel.setTitle("Menu");
		menuPanel.setCollapsible(true);
		menuPanel.setWidth(200);
		final AccordionLayout accordion = new AccordionLayout();
		accordion.setAnimate(true);
		menuPanel.setLayout(accordion);

		AbstractTreePanel reportsPanel = new ReportsMenuTreePanel();
		reportsPanel.setAdminModule(this);
		reportsPanel.setTitle("Reports");
		reportsPanel.setAutoScroll(true);
		reportsPanel.setBorder(false);
		reportsPanel.setIconCls("reports-menu-icon");
		menuPanel.add(reportsPanel);

		AbstractTreePanel settingsPanel = new SettingsMenuTreePanel();
		settingsPanel.setAdminModule(this);
		settingsPanel.setTitle("Settings");
		settingsPanel.setAutoScroll(true);
		settingsPanel.setBorder(false);
		settingsPanel.setIconCls("settings-menu-icon");
		menuPanel.add(settingsPanel);
		
		AbstractTreePanel schedulingPanel = new ScheduleMenuTreePanel();
		schedulingPanel.setAdminModule(this);
		schedulingPanel.setTitle("Scheduling");
		schedulingPanel.setAutoScroll(true);
		schedulingPanel.setBorder(false);
		schedulingPanel.setIconCls("settings-menu-icon");
		menuPanel.add(schedulingPanel);
		
        mainPanel.add(menuPanel, westLayoutData);

        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 0, 5, 5));

		final Panel welcomeTab = new Panel();
		welcomeTab.setTitle("Welcome");
		welcomeTab.setClosable(true);
		welcomeTab.setLayout(new FitLayout());
		welcomeTab.setAutoWidth(true);
		welcomeTab.setAutoScroll(true);
		Panel p = new Panel();
		p.setLayout(new FitLayout());
		String html = "<h2>Welcome to the AppSpy administration application !</h2>";
		html = html + "<br/>";
		html = html + "<p>Appspy is released under the <a target='_blank' href='http://www.apache.org/licenses/LICENSE-2.0.html'>Apache License, Version 2.0</a></p>";
		html = html + "<br/>";
		html = html + "<p>Copyright (c) 2008 <a target='_blank' href='http://www.appspy.org'>appspy.org</a></p>";
		p.setHtml(html);
		p.setBorder(false);
		p.setAutoWidth(true);
		p.setAutoScroll(true);
		welcomeTab.add(p);
		
        final Panel centerPanelWrappper = new Panel();
        centerPanelWrappper.setLayout(new FitLayout());
        centerPanelWrappper.setBorder(false);
        centerPanelWrappper.setBodyBorder(false);
//        centerPanelWrappper.setAutoWidth(true);
        centerPanelWrappper.setFrame(true);

        mTabPanel = new TabPanel();
        mTabPanel.setBodyBorder(false);
        mTabPanel.setEnableTabScroll(true);
        mTabPanel.setAutoScroll(true);
//        mTabPanel.setAutoWidth(true);
//        mTabPanel.setAutoDestroy(true);

        mTabPanel.setLayoutOnTabChange(true);
        mTabPanel.setTitle("Main Content");

		mTabPanel.add(welcomeTab);
		mTabPanel.setActiveTab(0);

//        centerPanelWrappper.add(mTabPanel);
        mainPanel.add(mTabPanel, centerLayoutData);

//        BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
//        eastLayoutData.setMargins(new Margins(0, 0, 0, 0));
//        
//        Panel rightPanel = new Panel();
//        rightPanel.setLayout(new FitLayout());
//        mainPanel.add(rightPanel, eastLayoutData);

        @SuppressWarnings("unused")
		Viewport viewport = new Viewport(mainPanel);

        mainPanel.doLayout();
        mainPanel.show();
	}
	
	/**
	 * @return the tabPanel
	 */
	public TabPanel getTabPanel() {
		return mTabPanel;
	}

	public AdminModuleServiceAsync getAdminModuleService() {
		if (mAdminModuleService == null)
		{
			mAdminModuleService = (AdminModuleServiceAsync) GWT
			.create(AdminModuleService.class);
			ServiceDefTarget endpoint = (ServiceDefTarget) mAdminModuleService;
			String moduleRelativeURL = GWT.getModuleBaseURL()
			+ "adminModuleService";
			endpoint.setServiceEntryPoint(moduleRelativeURL);
		}

		return mAdminModuleService;
	}
}
