/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.paramchooser;

import java.util.Date;
import java.util.HashMap;

import org.appspy.admin.client.util.Calendar;
import org.appspy.admin.client.util.GregorianCalendar;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.Widget;
import com.gwtext.client.util.DateUtil;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.form.DateField;
import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.TextArea;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.event.TextFieldListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.FormLayout;
import com.gwtext.client.widgets.tree.TreePanel;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DateChooser extends AbstractParamChooser {
	
	protected TreePanel mTreePanel = new TreePanel();

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#getChooserComponent()
	 */
	public Component getChooserComponent() {
		Panel p = new Panel();
		p.setFrame(true);
		p.setLayout(new FormLayout());
		
		TextArea textArea = new TextArea("Value");
		textArea.setHeight(150);
		p.add(textArea, new AnchorLayoutData("100%"));
		textArea.setValue(getValue().replaceAll(";", "\n"));
		
		final DateField dateField = new DateField("Pick a date for testing", "Y/m/d");
		p.add(dateField, new AnchorLayoutData("100%"));
		
		Button testButton = new Button("Test");
		p.add(testButton);
		
		final TextField dateTextField = new TextField("Test result");
		dateTextField.setReadOnly(true);
		p.add(dateTextField, new AnchorLayoutData("100%"));
		
		textArea.addListener(new TextFieldListenerAdapter(){

			@Override
			public void onChange(Field field, Object newVal, Object oldVal) {
				setValue(newVal.toString().replaceAll("\n", ";").replaceAll("\r", ""));
			}});
		
		testButton.addClickListener(new ClickListener(){

			/* (non-Javadoc)
			 * @see com.google.gwt.user.client.ui.ClickListener#onClick(com.google.gwt.user.client.ui.Widget)
			 */
			public void onClick(Widget sender) {
				dateTextField.setValue(computeDate(dateField.getValue()));
			}});
		
		dateField.setValue(new Date());
		return p;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#updateTextField(com.gwtext.client.widgets.form.TextField)
	 */
	public void updateTextField(final TextField textField) {
		textField.setValue(getValue());
	}
	
	public String computeDate(Date date)
	{
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(date);
		
		HashMap<String, Integer> fieldMap = new HashMap<String, Integer>();
		fieldMap.put("DATE", Calendar.DATE);
		fieldMap.put("DAY_OF_MONTH", Calendar.DAY_OF_MONTH);
		fieldMap.put("DAY_OF_WEEK", Calendar.DAY_OF_WEEK);
		fieldMap.put("DAY_OF_WEEK_IN_MONTH", Calendar.DAY_OF_WEEK_IN_MONTH);
		fieldMap.put("DAY_OF_YEAR", Calendar.DAY_OF_YEAR);
		fieldMap.put("HOUR", Calendar.HOUR);
		fieldMap.put("HOUR_OF_DAY", Calendar.HOUR_OF_DAY);
		fieldMap.put("MILLISECOND", Calendar.MILLISECOND);
		fieldMap.put("MINUTE", Calendar.MINUTE);
		fieldMap.put("MONTH", Calendar.MONTH);
		fieldMap.put("SECOND", Calendar.SECOND);
		fieldMap.put("WEEK_OF_MONTH", Calendar.WEEK_OF_MONTH);
		fieldMap.put("WEEK_OF_YEAR", Calendar.WEEK_OF_YEAR);
		fieldMap.put("YEAR", Calendar.YEAR);
		
		try
		{
			String[] lines = getValue().split(";");
			for (String line : lines) {
				String[] s = line.split(" ");
				String cmd = s[0];
				String field = s[1].toUpperCase();
				String val = s[2];
				
				if ("set".equalsIgnoreCase(cmd))
				{
					if (fieldMap.containsKey(field))
					{
						cal.set(fieldMap.get(field), val);
					}
				}
				else if ("add".equalsIgnoreCase(cmd))
				{
					if (fieldMap.containsKey(field))
					{
						cal.add(fieldMap.get(field), Integer.parseInt(val));
					}
				}
			}
		}
		catch (Exception ex)
		{
			GWT.log(ex.getMessage(), ex);
		}

		
		return DateUtil.format(cal.getTime(), "Y/m/d");
	}
}
