/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.paramchooser;

import java.util.Collection;
import java.util.TreeMap;

import org.appspy.admin.client.settings.form.EnvironmentForm;
import org.appspy.admin.client.settings.form.OrganizationForm;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.TreePanel;
import com.gwtext.client.widgets.tree.event.TreePanelListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class EnvironmentChooser extends AbstractParamChooser {
	
	protected TreePanel mTreePanel = new TreePanel();

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#getChooserComponent()
	 */
	public Component getChooserComponent() {
		mTreePanel.setTitle("Environments");
		mTreePanel.setAutoScroll(true);
		mTreePanel.setBorder(false);
		mTreePanel.setIconCls("settings-menu-icon");
		mTreePanel.setEnableDD(false);
		TreeNode root = new TreeNode("Environments");
		root.setExpanded(true);
		mTreePanel.setRootNode(root);
		mTreePanel.setRootVisible(false);
		
		getAdminModule().getAdminModuleService().getEnvironments(new AsyncCallback<Collection<EnvironmentForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<EnvironmentForm> result) {
				TreeMap<String, OrganizationForm> orgMap = new TreeMap<String, OrganizationForm>();
				TreeMap<String, EnvironmentForm> envMap = new TreeMap<String, EnvironmentForm>();
				for (EnvironmentForm env : result) {
					orgMap.put(env.getOrganization().getName(), env.getOrganization());
					envMap.put(env.getName(), env);
				}
				for (OrganizationForm org : orgMap.values()) {
					final TreeNode orgTreeNode = new TreeNode(org.getName());
					orgTreeNode.setUserObject(org);
					orgTreeNode.setIconCls("settings-menu-icon");
					mTreePanel.getRootNode().appendChild(orgTreeNode);
					for (EnvironmentForm env : envMap.values()) {
						if (env.getOrganization() == org)
						{
							final TreeNode envTreeNode = new TreeNode(env.getName());
							envTreeNode.setUserObject(env);
							envTreeNode.setIconCls("settings-menu-icon");
							orgTreeNode.appendChild(envTreeNode);
							if (env.getId().toString().equals(getValue()))
							{
								envTreeNode.ensureVisible();
								envTreeNode.select();
							}
						}
					}
				}
			}});
		
		mTreePanel.addListener(new TreePanelListenerAdapter(){

			@Override
			public void onClick(TreeNode node, EventObject e) {
				try
				{
					if (node.getUserObject() instanceof EnvironmentForm)
					{
						EnvironmentForm env = (EnvironmentForm) node.getUserObject();
						setValue(env.getId().toString());
					}
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}
			}});
		
		return mTreePanel;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#updateTextField(com.gwtext.client.widgets.form.TextField)
	 */
	public void updateTextField(final TextField textField) {
		getAdminModule().getAdminModuleService().getEnvironments(new AsyncCallback<Collection<EnvironmentForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<EnvironmentForm> result) {
				for (EnvironmentForm env : result) {
					if (env.getId().toString().equals(getValue()))
					{
						textField.setValue(env.getName());
					}
				}
			}});
	}
	
	
}
