/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.paramchooser;

import java.util.Collection;
import java.util.TreeMap;

import org.appspy.admin.client.settings.form.OrganizationForm;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.TreePanel;
import com.gwtext.client.widgets.tree.event.TreePanelListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class OrganizationChooser extends AbstractParamChooser {
	
	protected TreePanel mTreePanel = new TreePanel();

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#getChooserComponent()
	 */
	public Component getChooserComponent() {
		mTreePanel.setTitle("Organizations");
		mTreePanel.setAutoScroll(true);
		mTreePanel.setBorder(false);
		mTreePanel.setIconCls("settings-menu-icon");
		mTreePanel.setEnableDD(false);
		TreeNode root = new TreeNode("Organizations");
		root.setExpanded(true);
		mTreePanel.setRootNode(root);
		mTreePanel.setRootVisible(false);
		
		getAdminModule().getAdminModuleService().getOrganizations(new AsyncCallback<Collection<OrganizationForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<OrganizationForm> result) {
				TreeMap<String, OrganizationForm> appMap = new TreeMap<String, OrganizationForm>();
				for (OrganizationForm org : result) {
					appMap.put(org.getName(), org);
				}
				for (OrganizationForm org : appMap.values()) {
					TreeNode treeNode = new TreeNode(org.getName());
					treeNode.setUserObject(org);
					treeNode.setIconCls("settings-menu-icon");
					mTreePanel.getRootNode().appendChild(treeNode);
					if (org.getId().toString().equals(getValue()))
					{
						treeNode.select();
					}
				}
			}});
		
		mTreePanel.addListener(new TreePanelListenerAdapter(){

			@Override
			public void onClick(TreeNode node, EventObject e) {
				try
				{
					OrganizationForm org = (OrganizationForm) node.getUserObject();
					setValue(org.getId().toString());
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}
			}});
		
		return mTreePanel;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#updateTextField(com.gwtext.client.widgets.form.TextField)
	 */
	public void updateTextField(final TextField textField) {
		getAdminModule().getAdminModuleService().getOrganizations(new AsyncCallback<Collection<OrganizationForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<OrganizationForm> result) {
				for (OrganizationForm org : result) {
					if (org.getId().toString().equals(getValue()))
					{
						textField.setValue(org.getName());
					}
				}
			}});
	}
	
	
}
