/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.reports;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.TreeMap;
import java.util.Map.Entry;

import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.reports.form.DashboardColumnForm;
import org.appspy.admin.client.reports.form.DashboardForm;
import org.appspy.admin.client.reports.form.DashboardParamBindingForm;
import org.appspy.admin.client.reports.form.DashboardReportForm;
import org.appspy.admin.client.reports.form.ReportCategoryForm;
import org.appspy.admin.client.reports.form.ReportForm;
import org.appspy.admin.client.reports.form.ReportParamForm;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Function;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.Node;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.dd.DragData;
import com.gwtext.client.dd.DragSource;
import com.gwtext.client.dd.DropTarget;
import com.gwtext.client.dd.DropTargetConfig;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.Container;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.Tool;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.event.ContainerListenerAdapter;
import com.gwtext.client.widgets.form.ComboBox;
import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.FieldSet;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.NumberField;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.event.TextFieldListenerAdapter;
import com.gwtext.client.widgets.layout.AccordionLayout;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.ColumnLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtext.client.widgets.portal.Portal;
import com.gwtext.client.widgets.portal.PortalColumn;
import com.gwtext.client.widgets.portal.Portlet;
import com.gwtext.client.widgets.tree.TreeDragData;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.TreePanel;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DashboardComposer extends Panel {

	protected DashboardForm mDashboard = null;
	protected AdminModule mAdminModule = null;

	protected TreePanel mReportsTreePanel = new TreePanel();

	protected TreeMap<DashboardColumnForm, PortalColumn> mColumnMap = new TreeMap<DashboardColumnForm, PortalColumn>();

	protected TreeMap<PortalColumn, ArrayList<DashboardReportForm>> mDashboardReports = new TreeMap<PortalColumn, ArrayList<DashboardReportForm>>();

	protected Portal mPortal = null;

	public DashboardComposer(AdminModule adminModule, DashboardForm dashboard) {
		mAdminModule = adminModule;
		mDashboard = dashboard;

		setLayout(new BorderLayout());

		// setup the west regions layout properties
		BorderLayoutData westLayoutData = new BorderLayoutData(RegionPosition.WEST);
		westLayoutData.setMargins(new Margins(5, 5, 0, 5));
		westLayoutData.setCMargins(new Margins(5, 5, 5, 5));
		westLayoutData.setMinSize(155);
		westLayoutData.setMaxSize(350);
		westLayoutData.setSplit(true);

		BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
		centerLayoutData.setMargins(new Margins(5, 0, 5, 5));

		Panel westPanel = new Panel();
		westPanel.setCollapsible(true);
		westPanel.setWidth(200);
		final AccordionLayout accordion = new AccordionLayout();
		accordion.setAnimate(true);
		westPanel.setLayout(accordion);

		mReportsTreePanel.setTitle("Reports");
		mReportsTreePanel.setAutoScroll(true);
		mReportsTreePanel.setBorder(false);
		mReportsTreePanel.setIconCls("reports-menu-icon");
		mReportsTreePanel.setEnableDD(true);
		mReportsTreePanel.setDdGroup("DragReport");
		westPanel.add(mReportsTreePanel);
		TreeNode root = new TreeNode("Reports");
		root.setExpanded(true);
		mReportsTreePanel.setRootNode(root);
		mReportsTreePanel.setRootVisible(false);

		add(westPanel, westLayoutData);

		mPortal = new Portal();
		mPortal.setBodyBorder(false);
		mPortal.setAutoScroll(true);
		mPortal.setTitle("Dashboard disposition");
		mPortal.setDraggable(true);

		Toolbar toolbar = new Toolbar();
		toolbar.addButton(new ToolbarButton("Add Column", new ButtonListenerAdapter() {

			@Override
			public void onClick(Button button, EventObject e) {
				int width = getDashboardWidth() + 1;
				for (Component c : mPortal.getComponents()) {
					mPortal.remove(c, false);
					DashboardPortalColumn col = (DashboardPortalColumn) c;
					GWT.log("add layout = " + ((double) col.getDashboardColumn().getWidth()), null);
					mPortal.add(c, new ColumnLayoutData(((double) col.getDashboardColumn().getWidth()) / width));
				}
				DashboardColumnForm dashboardColumn = new DashboardColumnForm();
				dashboardColumn.setWidth(1);
				dashboardColumn.setDashboard(mDashboard);
				dashboardColumn.setIndex(mPortal.getComponents().length);
				mDashboard.getColumns().add(dashboardColumn);
				addColumn(dashboardColumn);
			}
		}));

		toolbar.addButton(new ToolbarButton("Remove Column", new ButtonListenerAdapter() {

			@Override
			public void onClick(Button button, EventObject e) {
				DashboardPortalColumn col = (DashboardPortalColumn) mPortal.getComponent(mPortal.getComponents().length - 1);
				int width = getDashboardWidth() - col.getDashboardColumn().getWidth();
				mPortal.remove(col, true);
				mDashboard.getColumns().remove(col.getDashboardColumn());
				for (Component c : mPortal.getComponents()) {
					mPortal.remove(c, false);
					DashboardPortalColumn column = (DashboardPortalColumn) c;
					double w = ((double) column.getDashboardColumn().getWidth()) / width;
					mPortal.add(c, new ColumnLayoutData(w));
				}

				mPortal.doLayout();
			}
		}));
		toolbar.addButton(new ToolbarButton("Column Width", new ButtonListenerAdapter() {

			@Override
			public void onClick(Button button, EventObject e) {
				final Window w = new Window();
				w.setModal(true);
				w.setFrame(true);
				w.setLayout(new FitLayout());
				w.setSize(400, 200);
				w.setTitle("Edit column weights");

				FormPanel formPanel = new FormPanel();
				formPanel.setFrame(true);

				int i = 0;
				for (Component c : mPortal.getComponents()) {
					final DashboardPortalColumn col = (DashboardPortalColumn) c;
					// Panel p = new Panel();
					final TextField tf = new TextField("Column " + ++i);
					tf.setValue(col.getDashboardColumn().getWidth().toString());
					formPanel.add(tf, new AnchorLayoutData("100%"));
					tf.addListener(new TextFieldListenerAdapter() {
						@Override
						public void onChange(Field field, Object newVal, Object oldVal) {
							col.getDashboardColumn().setWidth(Integer.valueOf(tf.getValueAsString()));
						}
					});
				}

				formPanel.addButton(new Button("OK", new ButtonListenerAdapter() {

					/*
					 * (non-Javadoc)
					 * 
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button,
					 *      com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						int width = getDashboardWidth();
						for (Component c : mPortal.getComponents()) {
							mPortal.remove(c, false);
							DashboardPortalColumn column = (DashboardPortalColumn) c;
							double w = ((double) column.getDashboardColumn().getWidth()) / width;
							mPortal.add(c, new ColumnLayoutData(w));
						}

						mPortal.doLayout();
						w.close();
						w.destroy();
					}
				}));

				w.add(formPanel);
				w.show();
			}
		}));
		mPortal.setTopToolbar(toolbar);

		if ((mDashboard.getColumns() == null) || (mDashboard.getColumns().size() == 0)) {
			try {
				DashboardColumnForm dashboardColumn = new DashboardColumnForm();
				dashboardColumn.setWidth(1);
				dashboardColumn.setDashboard(mDashboard);
				dashboardColumn.setIndex(0);
				mDashboard.getColumns().add(dashboardColumn);
			} catch (Exception ex) {
				GWT.log("pb : ", ex);
			}
		}
		TreeMap<Integer, DashboardColumnForm> columnTreeMap = new TreeMap<Integer, DashboardColumnForm>();
		for (DashboardColumnForm dashboardColumn : mDashboard.getColumns()) {
			columnTreeMap.put(dashboardColumn.getIndex(), dashboardColumn);
		}
		for (DashboardColumnForm dashboardColumn : columnTreeMap.values()) {
			addColumn(dashboardColumn);
		}

		add(mPortal, centerLayoutData);
		
		refreshDatas();
	}

	public void refreshDatas() {
		final AsyncCallback<Collection<ReportForm>> reportCallback = new AsyncCallback<Collection<ReportForm>>() {
			public void onSuccess(Collection<ReportForm> reports) {

				Node[] childs = mReportsTreePanel.getRootNode().getChildNodes();
				for (Node node : childs) {
					node.remove();
				}
				
				HashMap<Long, TreeNode> cats = new HashMap<Long, TreeNode>();

				for (ReportForm report : reports) {

					TreeNode parent = cats.get(report.getCategory().getId());
					if (parent == null)
					{
						parent = createCategoryTreeNode(report.getCategory());
						mReportsTreePanel.getRootNode().appendChild(parent);
						cats.put(report.getCategory().getId(), parent);
					}
					TreeNode reportNode = new TreeNode(report.getName());
					reportNode.setAllowDrag(true);
					reportNode.setIconCls("reports-menu-icon");
					reportNode.setUserObject(report);
					parent.appendChild(reportNode);
				}
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		mAdminModule.getAdminModuleService().getReports(reportCallback);
	}
	
	public TreeNode createCategoryTreeNode(ReportCategoryForm cat)
	{
		TreeNode categoryTreeNode = new TreeNode(cat.getName());
		return categoryTreeNode;
	}

	public void addColumn(final DashboardColumnForm dashboardColumn) {
		final PortalColumn portalColumn = new DashboardPortalColumn(dashboardColumn);
		portalColumn.setPaddings(10, 10, 0, 10);

		TreeMap<Integer, DashboardReportForm> reportTreeMap = new TreeMap<Integer, DashboardReportForm>();
		for (DashboardReportForm dashboardReport : dashboardColumn.getDashboardReports()) {
			reportTreeMap.put(dashboardReport.getRowIndex(), dashboardReport);
		}
		for (DashboardReportForm dashboardReport : reportTreeMap.values()) {
			Portlet portlet = new DashboardReportPortlet(dashboardReport);
			portalColumn.add(portlet);
		}

		final Portlet freePortlet = new Portlet();
		freePortlet.setTitle("Free slot");
		Panel freePanel = new Panel();
		freePanel.setHtml("Drop Report Here !");
		freePortlet.add(freePanel);
		portalColumn.add(freePortlet);
		freePortlet.setDraggable(false);
		freePortlet.setCollapsible(false);
		freePortlet.setBorder(false);
		freePortlet.setBodyBorder(false);

		// mDashboardReports.put(portalColumn, dashboardReports);

		double columnWidth = dashboardColumn.getWidth();
		GWT.log("addColumn layout = " + columnWidth, null);
		mPortal.add(portalColumn, new ColumnLayoutData(columnWidth / getDashboardWidth()));
		// mColumnMap.put(dashboardColumn, portalColumn);

		portalColumn.addListener(new ContainerListenerAdapter() {

			@Override
			public void onAdd(Container self, Component component, int index) {
				if (component instanceof DashboardReportPortlet) {
					
					DashboardReportPortlet dashboardReportPortlet = (DashboardReportPortlet) component;
					dashboardColumn.getDashboardReports().add(dashboardReportPortlet.getDashboardReport());
					dashboardReportPortlet.getDashboardReport().setDashboardColumn(dashboardColumn);
					
					updateDashboardReportIndexes();
				}
			}

			@Override
			public void onRemove(Container self, Component component) {
				if (component instanceof DashboardReportPortlet) {
					
					DashboardReportPortlet dashboardReportPortlet = (DashboardReportPortlet) component;
					dashboardColumn.getDashboardReports().remove(dashboardReportPortlet.getDashboardReport());
					
					updateDashboardReportIndexes();
				}
			}
			
			public void updateDashboardReportIndexes()
			{
				int i = 0;
				for (Component component : portalColumn.getComponents()) {
					if (component instanceof DashboardReportPortlet)
					{
						DashboardReportPortlet dashboardReportPortlet = (DashboardReportPortlet) component;
						dashboardReportPortlet.getDashboardReport().setRowIndex(i++);
					}
				}
			}
		});

		DropTargetConfig dropConfig = new DropTargetConfig();
		dropConfig.setdDdGroup("DragReport");
		@SuppressWarnings("unused")
		DropTarget dFlow = new DropTarget(freePanel, dropConfig) {
			public String notifyEnter(DragSource src, EventObject e, DragData data) {
				// GWT.log("notifyEnter", new Exception());
				return "x-tree-drop-ok-append";
			}

			public String notifyOver(DragSource src, EventObject e, DragData data) {
				// GWT.log("notifyOver", new Exception());
				return "x-tree-drop-ok-append";
			}

			public boolean notifyDrop(DragSource src, EventObject e, DragData data) {
				GWT.log("notifyDrop", null);
				GWT.log("dragData = " + data.getClass().getName(), null);
				if (data instanceof TreeDragData) {
					TreeDragData treeDragData = (TreeDragData) data;
					TreeNode treeNode = treeDragData.getTreeNode();
					GWT.log("dragTreeNode = " + treeNode.getClass().getName(), null);
					GWT.log("dragTreeNode.text = " + treeNode.getText(), null);
					GWT.log("tn.getUserObject = " + treeNode.getUserObject().getClass().getName(), null);
					if (treeNode.getUserObject() != null && treeNode.getUserObject() instanceof ReportForm) {
						int rowIndex = portalColumn.getComponents().length - 1;

						ReportForm report = (ReportForm) treeNode.getUserObject();

						DashboardReportForm dashboardReport = new DashboardReportForm();
						dashboardReport.setDashboardColumn(dashboardColumn);
						// dashboardReport.setDashboardParamBindings(dashboardParamBindings);
						dashboardReport.setHeight(100);
						dashboardReport.setTitle(report.getName());
						dashboardReport.setReport(report);
						dashboardReport.setRowIndex(rowIndex);
//						dashboardColumn.getDashboardReports().add(dashboardReport);

						final DashboardReportPortlet p = new DashboardReportPortlet(dashboardReport);
						// p.setHtml("html");
						portalColumn.insert(rowIndex, p);
						p.doLayout();
						portalColumn.doLayout();
						mPortal.doLayout();
						return true;
					}
				}
				return false;
			}
		};

		mPortal.doLayout();
	}

	public int getDashboardWidth() {
		int width = 0;

		for (DashboardColumnForm dashboardColumn : mDashboard.getColumns()) {
			width = width + dashboardColumn.getWidth();
		}

		return width;
	}
}

class DashboardReportPortlet extends Portlet {
	protected DashboardReportForm mDashboardReport;

	public DashboardReportPortlet(DashboardReportForm dashboardReport) {
		mDashboardReport = dashboardReport;
		
		final DashboardReportPortlet p = this;

		setTitle(dashboardReport.getTitle());
		setHeight(dashboardReport.getHeight());
		Tool gear = new Tool(Tool.GEAR, new Function() {
			public void execute() {
				final Window w = new Window();
				w.setModal(true);
				w.setFrame(true);
				w.setLayout(new FitLayout());
				w.setSize(600, 500);
				w.setTitle("Report configuration");

				FormPanel formPanel = new FormPanel();
				formPanel.setFrame(true);
				formPanel.setAutoScroll(true);

				final TextField titleTextField = new TextField("Title");
				titleTextField.setValue(mDashboardReport.getTitle());
				formPanel.add(titleTextField, new AnchorLayoutData("100%"));

				final TextField heightTextField = new NumberField("Height");
				heightTextField.setValue(mDashboardReport.getHeight().toString());
				formPanel.add(heightTextField, new AnchorLayoutData("100%"));
				
				FieldSet paramBindingFieldSet = new FieldSet("Parameter bindings");  
				paramBindingFieldSet.setCollapsible(true);  
				paramBindingFieldSet.setAutoHeight(true);
				
				RecordDef dashboardParamsRecordDef = new RecordDef(new FieldDef[]{
						new ObjectFieldDef("object"),
						new IntegerFieldDef("id"),
						new StringFieldDef("name")
				});
				final Store dashboardParamsStore = new Store(dashboardParamsRecordDef);
				for (ReportParamForm param : mDashboardReport.getDashboardColumn().getDashboard().getParams()) {
					dashboardParamsStore.add(dashboardParamsRecordDef.createRecord(new Object[]{
							param,
							param.getId(),
							param.getName()
					}));
				}

				final HashMap<ComboBox, DashboardParamBindingForm> comboParamBindingMap = new HashMap<ComboBox, DashboardParamBindingForm>();
				for (ReportParamForm reportParam : mDashboardReport.getReport().getParams()) {
			        final ComboBox paramComboBox = new ComboBox(reportParam.getName());
			        paramBindingFieldSet.add(paramComboBox, new AnchorLayoutData("100%"));
			        paramComboBox.setStore(dashboardParamsStore);
			        paramComboBox.setValueField("id");
			        paramComboBox.setDisplayField("name");
			        paramComboBox.setMode(ComboBox.LOCAL);
			        paramComboBox.setForceSelection(true);
			        paramComboBox.setDisableKeyFilter(true);
			        paramComboBox.setLazyRender(true);
			        paramComboBox.setTriggerAction(ComboBox.ALL);
			        
			        for (DashboardParamBindingForm paramBinding : mDashboardReport.getDashboardParamBindings()) {
						if (paramBinding.getReportParam() != null && paramBinding.getReportParam().getId() == reportParam.getId())
						{
							try
							{
								comboParamBindingMap.put(paramComboBox, paramBinding);
								paramComboBox.setValue(paramBinding.getDashboardParam().getId().toString());
							}
							catch (Exception ex)
							{
							}
						}
					}
			        
			        if (!comboParamBindingMap.containsKey(paramComboBox))
			        {
			        	DashboardParamBindingForm paramBinding = new DashboardParamBindingForm();
			        	paramBinding.setDashboardReport(mDashboardReport);
			        	paramBinding.setReportParam(reportParam);
			        	comboParamBindingMap.put(paramComboBox, paramBinding);
			        }
				}
				
				formPanel.add(paramBindingFieldSet, new AnchorLayoutData("100%"));

				formPanel.addButton(new Button("OK", new ButtonListenerAdapter() {

					/*
					 * (non-Javadoc)
					 * 
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button,
					 *      com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						
						mDashboardReport.setTitle(titleTextField.getValueAsString());
						setTitle(mDashboardReport.getTitle());
						
						mDashboardReport.setHeight(Integer.valueOf(heightTextField.getValueAsString()));
						setHeight(mDashboardReport.getHeight());
						
						mDashboardReport.getDashboardParamBindings().clear();
						for (Entry<ComboBox, DashboardParamBindingForm> entry : comboParamBindingMap.entrySet()) {
							DashboardParamBindingForm paramBinding = entry.getValue();
							paramBinding.setDashboardParam((ReportParamForm) dashboardParamsStore.query("id",	entry.getKey().getValue())[0].getAsObject("object"));
							mDashboardReport.getDashboardParamBindings().add(paramBinding);
						}
						
						w.close();
						w.destroy();
					}
				}));

				w.add(formPanel);
				w.show();
			}
		});
		
		Tool close = new Tool(Tool.CLOSE, new Function() {
			public void execute() {
				p.getOwnerContainer().remove(p);
			}
		});
		
		Tool[] tools = new Tool[] { gear, close };
		
		setTools(tools);
	}

	/**
	 * @return the dashboardReport
	 */
	public DashboardReportForm getDashboardReport() {
		return mDashboardReport;
	}

	/**
	 * @param dashboardReport
	 *            the dashboardReport to set
	 */
	public void setDashboardReport(DashboardReportForm dashboardReport) {
		mDashboardReport = dashboardReport;
	}

}

class DashboardPortalColumn extends PortalColumn {
	protected DashboardColumnForm mDashboardColumn;

	public DashboardPortalColumn(DashboardColumnForm dashboardColumn) {
		mDashboardColumn = dashboardColumn;
	}

	/**
	 * @return the dashboardColumn
	 */
	public DashboardColumnForm getDashboardColumn() {
		return mDashboardColumn;
	}

	/**
	 * @param dashboardColumn
	 *            the dashboardColumn to set
	 */
	public void setDashboardColumn(DashboardColumnForm dashboardColumn) {
		mDashboardColumn = dashboardColumn;
	}
}
