/**
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 *
 * You may obtain a copy of the License at 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */
package org.appspy.admin.client.reports;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.TreeMap;

import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.reports.form.DashboardForm;
import org.appspy.admin.client.reports.form.ReportCategoryForm;
import org.appspy.admin.client.reports.form.ReportForm;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.NamedFrame;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.Widget;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.TreePanel;
import com.gwtext.client.widgets.tree.event.TreePanelListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ExportWindow extends Window
{
	public ExportWindow(final AdminModule adminModule)
	{
		setTitle("Export dashboards and reports");
		setClosable(true);
		setFrame(true);
		setWidth(600);
		setHeight(400);
		setAutoScroll(true);
		setPlain(true);
		setLayout(new FitLayout());
		setCloseAction(Window.CLOSE);
		setModal(true);

		TreePanel tp = new TreePanel();
		tp.setAutoScroll(true);
		tp.setRootVisible(false);
		final TreeNode root = new TreeNode("root");
		root.setChecked(false);
		tp.setRootNode(root);

		final TreeMap<String, ReportForm> availableReports = new TreeMap<String, ReportForm>();
		final AsyncCallback<Collection<ReportForm>> reportCallback = new AsyncCallback<Collection<ReportForm>>() {
			public void onSuccess(Collection<ReportForm> reports) {

				TreeMap<String, ReportCategoryForm> cats = new TreeMap<String, ReportCategoryForm>();
				for (ReportForm report : reports) {
					availableReports.put(report.getName(), report);
					if (!cats.containsKey(report.getCategory().getName()))
					{
						cats.put(report.getCategory().getName(), report.getCategory());
					}
				}
				
				TreeNode reportsNode = new TreeNode("reports");
				root.appendChild(reportsNode);
				
				for (ReportCategoryForm cat : cats.values())
                {
					TreeNode catNode = new TreeNode(cat.getName());
	                reportsNode.appendChild(catNode);
	                
	                for (ReportForm report : availableReports.values())
                    {
	                    if (report.getCategory().getName().equals(cat.getName()))
	                    {
	                    	TreeNode reportNode = new TreeNode(report.getName());
	                    	reportNode.setUserObject(report);
	                    	reportNode.setChecked(false);
	                    	catNode.appendChild(reportNode);
	                    }
                    }
                }
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};
		adminModule.getAdminModuleService().getReports(reportCallback);
		
		final TreeMap<String, DashboardForm> availableDashboards = new TreeMap<String, DashboardForm>();
		final AsyncCallback<Collection<DashboardForm>> dashboardCallback = new AsyncCallback<Collection<DashboardForm>>() {
			public void onSuccess(Collection<DashboardForm> dashboards) {

				TreeMap<String, ReportCategoryForm> cats = new TreeMap<String, ReportCategoryForm>();
				for (DashboardForm dashboard : dashboards) {
					availableDashboards.put(dashboard.getName(), dashboard);
					if (!cats.containsKey(dashboard.getCategory().getName()))
					{
						cats.put(dashboard.getCategory().getName(), dashboard.getCategory());
					}
				}
				
				TreeNode dashboardsNode = new TreeNode("dashboards");
				root.appendChild(dashboardsNode);
				
				for (ReportCategoryForm cat : cats.values())
                {
					TreeNode catNode = new TreeNode(cat.getName());
	                dashboardsNode.appendChild(catNode);
	                
	                for (DashboardForm dashboard : availableDashboards.values())
                    {
	                    if (dashboard.getCategory().getName().equals(cat.getName()))
	                    {
	                    	TreeNode dashboardNode = new TreeNode(dashboard.getName());
	                    	dashboardNode.setUserObject(dashboard);
	                    	dashboardNode.setChecked(false);
	                    	catNode.appendChild(dashboardNode);
	                    }
                    }
                }
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};
		adminModule.getAdminModuleService().getDashboards(dashboardCallback);
		
		final List<DashboardForm> selectedDashboards = new ArrayList<DashboardForm>();
		final List<ReportForm> selectedReports = new ArrayList<ReportForm>();
		
		tp.addListener(new TreePanelListenerAdapter(){

            @Override
            public void onCheckChange(TreeNode node, boolean checked)
            {
	            if (node.getUserObject() instanceof DashboardForm)
	            {
	            	DashboardForm d = (DashboardForm) node.getUserObject();
	            	GWT.log(d.getName() + " is checked ? " + checked, null);
	            	if (checked)
	            	{
	            		selectedDashboards.add(d);
	            	}
	            	else
	            	{
	            		selectedDashboards.remove(d);
	            	}
	            }
	            else if (node.getUserObject() instanceof ReportForm)
	            {
	            	ReportForm r = (ReportForm) node.getUserObject();
	            	GWT.log(r.getName() + " is checked ? " + checked, null);
	            	if (checked)
	            	{
	            		selectedReports.add(r);
	            	}
	            	else
	            	{
	            		selectedReports.remove(r);
	            	}
	            }
            }});

		add(tp);
		
        final Button downloadButton = new Button("Download");
        tp.addButton(downloadButton);
        downloadButton.addListener(new ButtonListenerAdapter(){
			@Override
			public void onClick(Button button, EventObject e) {
				
		        boolean frameExists = (RootPanel.get("downloadiframe") != null);
		        if(frameExists) {
		            Widget widgetFrame = (Widget)RootPanel.get("downloadiframe");
		            widgetFrame.removeFromParent();
		        }
		        
		        String frameUrl = GWT.getModuleBaseURL() + "exportReportServlet?reportIds=" + getReportIds() + "&dashboardIds=" + getDashboardIds();
		        GWT.log("Frame url = " + frameUrl, null);
		        NamedFrame frame = new NamedFrame("downloadiframe");
		        frame.setUrl(frameUrl);
		        frame.setVisible(false);
		        
		        RootPanel.get().add(frame);
			}

			private String getDashboardIds()
            {
	            String result = null;
	            
	            for (DashboardForm dashboardForm : selectedDashboards)
                {
	            	if (result == null)
	            	{
	            		result = dashboardForm.getId().toString();
	            	}
	            	else
	            	{
	            		result = result + "," + dashboardForm.getId().toString();
	            	}
                }
	            
	            if (result == null)
	            {
	            	result = "";
	            }

	            return result;
            }

			private String getReportIds()
            {
	            String result = null;
	            
	            for (ReportForm reportForm : selectedReports)
                {
	            	if (result == null)
	            	{
	            		result = reportForm.getId().toString();
	            	}
	            	else
	            	{
	            		result = result + "," + reportForm.getId().toString();
	            	}
                }
	            
	            if (result == null)
	            {
	            	result = "";
	            }
	            
	            return result;
            }});

	}
}
