/**
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 *
 * You may obtain a copy of the License at 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */
package org.appspy.admin.client.reports;

import org.appspy.admin.client.AdminModule;

import com.gwtext.client.core.EventObject;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.data.XmlReader;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.Form;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.event.FormListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ImportWindow extends Window
{
	public ImportWindow(final AdminModule adminModule)
	{
		setTitle("Import dashboards and reports");
		setClosable(true);
		setFrame(true);
		setWidth(600);
		setHeight(400);
		setAutoScroll(true);
		setPlain(true);
		setLayout(new FitLayout());
		setCloseAction(Window.CLOSE);
		setModal(true);

		final FormPanel formPanel = new FormPanel();
		formPanel.setPaddings(10, 10, 10, 10);
		formPanel.setFrame(true);

        final TextField fileField = new TextField("File");
        fileField.setInputType("file");
        fileField.setName("importFile");
        formPanel.add(fileField, new AnchorLayoutData("100%"));
        formPanel.setUrl("importReportServlet");
        formPanel.setFileUpload(true);

        final XmlReader reader = new XmlReader("data", new RecordDef(new
        		FieldDef[] { new StringFieldDef("responseMsg") }));
        		        reader.setSuccess("@success");

        final XmlReader errorReader = new XmlReader("data", new
        		RecordDef(new FieldDef[] { new StringFieldDef("responseMsg") }));
        		        errorReader.setSuccess("@success");
        
        formPanel.setReader(reader);
        formPanel.setErrorReader(errorReader);
        
        final Button createButton = new Button("Import");
		formPanel.addButton(createButton);
		
		final Form form = formPanel.getForm();
		form.addListener(new FormListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.form.event.FormListenerAdapter#onActionComplete(com.gwtext.client.widgets.form.Form, int, java.lang.String)
			 */
			@Override
			public void onActionComplete(Form form, int httpStatus, String responseText) {
				close();
			}

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.form.event.FormListenerAdapter#onActionFailed(com.gwtext.client.widgets.form.Form, int, java.lang.String)
			 */
			@Override
			public void onActionFailed(Form form, int httpStatus, String responseText) {
				MessageBox.alert("Problem during import. See server logs for details.");
			}});

		createButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
						form.submit("importReportServlet");
					}
			});
		
		add(formPanel);
	}
}
