/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.reports;

import java.util.Collection;

import org.appspy.admin.client.AbstractPanel;
import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.reports.form.DashboardForm;
import org.appspy.admin.client.reports.form.ReportCategoryForm;
import org.appspy.admin.client.reports.form.ReportParamForm;
import org.appspy.admin.client.reports.i18n.ReportMessages;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.data.BooleanFieldDef;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.ToolTip;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.Checkbox;
import com.gwtext.client.widgets.form.ComboBox;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtextux.client.widgets.grid.plugins.GridSearchPlugin;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListDashboardPanel extends AbstractPanel {

	public static final String TAB_ID = "ListDashboardPanel";
	
	public static ReportMessages sMessages = GWT.create(ReportMessages.class);
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[] { 
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name"), 
			new BooleanFieldDef("hidden"),
			new StringFieldDef("catName"),
			new IntegerFieldDef("catId")});
	
	protected RecordDef mCatRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name")});
	
	protected EditorGridPanel mGridPanel = null;
	protected FormPanel mFormPanel = null;
	
	protected Store mStore = null;
	protected Store mCatStore = null;
	
	public ListDashboardPanel(AdminModule adminModule) {
		
		super(adminModule);

		setTitle("Dashboards");

		setLayout(new BorderLayout());
		setAutoScroll(true);

		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig(sMessages.dashboardName(), "name", 100, true);
		ColumnConfig hiddenColumnConfig = new ColumnConfig("HIDDEN", "hidden", 30, true);
		ColumnConfig catNameColumnConfig = new ColumnConfig("CATEGORY", "catName", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				nameColumnConfig,
				hiddenColumnConfig,
				catNameColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);

		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(1);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		mStore = new Store(mRecordDef);
		mGridPanel.setStore(mStore);
		
		mCatStore = new Store(mCatRecordDef);
		
		Toolbar toolbar = new Toolbar();
		ToolbarButton newButton = new ToolbarButton("Add", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New dashboard");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(600);
				window.setHeight(400);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);
				
				final DashboardForm dashboard = new DashboardForm();
				final Record record = mRecordDef.createRecord(new Object[]{
						dashboard,
						null, 
						"", 
						false,
						"",
						null});
				
				TextField nameTextField = new TextField(sMessages.dashboardName(), "name");
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));
				ToolTip fieldToolTip = new ToolTip("The name of the dashboard.");  
				fieldToolTip.applyTo(nameTextField);
				
				Checkbox hiddenCheckBox = new Checkbox("Hidden", "hidden");
				formPanel.add(hiddenCheckBox, new AnchorLayoutData("100%"));
				
		        final ComboBox catComboBox = new ComboBox("Category", "catId");
		        formPanel.add(catComboBox, new AnchorLayoutData("100%"));
		        catComboBox.setStore(mCatStore);
		        catComboBox.setValueField("id");
		        catComboBox.setDisplayField("name");
		        catComboBox.setMode(ComboBox.LOCAL);
		        catComboBox.setForceSelection(true);
		        catComboBox.setDisableKeyFilter(true);
		        catComboBox.setLazyRender(true);
		        catComboBox.setTriggerAction(ComboBox.ALL);

		        final ListParamPanel listParamPanel = new ListParamPanel(getAdminModule());
		        formPanel.add(listParamPanel, new AnchorLayoutData("100% -70"));

		        final Button createButton = new Button("Create");
				formPanel.addButton(createButton);
				
				formPanel.getForm().loadRecord(record);

				createButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						dashboard.setName(record.getAsString("name"));
						dashboard.setHidden(record.getAsBoolean("hidden"));
						ReportCategoryForm cat = (ReportCategoryForm) mCatStore.query("id", catComboBox.getValueAsString())[0].getAsObject("object");
						record.set("catName", cat.getName());
						dashboard.setCategory(cat);
						dashboard.setParams(listParamPanel.getParameters());
						for (ReportParamForm param : listParamPanel.getParameters()) {
							param.setDashboard(dashboard);
						}
						final AsyncCallback<DashboardForm> callback = new AsyncCallback<DashboardForm>(){

							public void onFailure(Throwable caught) {
								MessageBox.alert("Technical problem", "Unable to create dashboard");
							}

							public void onSuccess(DashboardForm result) {
								record.set("object", result);
								record.set("id", result.getId());
								mStore.add(record);
								record.commit();
								
								window.close();
								
							}
						};
						
						getAdminModule().getAdminModuleService().saveDashboard(dashboard, callback);
					}});
				
				window.add(formPanel);
				window.show();
			}
		});
		newButton.setIcon("images/silk/add.gif");
		toolbar.addButton(newButton);

		toolbar.addSeparator();

		ToolbarButton delButton = new ToolbarButton("Del", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				try {
					final Record record = mGridPanel.getSelectionModel().getSelected();
					if (record != null)
					{
						MessageBox.confirm("Confirm", "Are you sure you want to delete this dashboard ?",
								new MessageBox.ConfirmCallback() {

									public void execute(String btnID) {
										if (btnID.equals("yes")) {
											DashboardForm dashboard = (DashboardForm) record.getAsObject("object");
											getAdminModule().getAdminModuleService().deleteDashboard(dashboard, new AsyncCallback<DashboardForm>(){

												public void onFailure(Throwable caught) {
													MessageBox.alert("Unable to delete dashboard", caught.getLocalizedMessage());
												}

												public void onSuccess(DashboardForm result) {
													mStore.remove(record);
												}});
										}
									}
								});
					}
				} catch (Exception ex) {
					MessageBox.alert(ex.getMessage());
				}
			}
		});
		delButton.setIcon("images/silk/delete.gif");
		toolbar.addButton(delButton);
		
		toolbar.addSeparator();
		toolbar.addSpacer();
		
		ToolbarButton refreshButton = new ToolbarButton("Refresh", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				refreshDatas();
			}});
		refreshButton.setIcon("images/icons/fam/table_refresh.png");
		toolbar.addButton(refreshButton);

		ToolbarButton exportButton = new ToolbarButton("Export", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				ExportWindow window = new ExportWindow(getAdminModule());
				
				window.show();
				
			}});
		toolbar.addButton(exportButton);

		ToolbarButton importButton = new ToolbarButton("Import", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				ImportWindow window = new ImportWindow(getAdminModule());
				
				window.show();
				
			}});
		toolbar.addButton(importButton);

		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		GridSearchPlugin gridSearch = new GridSearchPlugin(GridSearchPlugin.TOP);  
		gridSearch.setMode(GridSearchPlugin.LOCAL);  
		mGridPanel.addPlugin(gridSearch);  

        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 5, 0, 5));
        centerLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        centerLayoutData.setMinSize(200);
        centerLayoutData.setMinWidth(200);
        
        add(mGridPanel, centerLayoutData);

        mFormPanel = new FormPanel();
        mFormPanel.setPaddings(10, 10, 10, 10);
        mFormPanel.setFrame(true);
        mFormPanel.setAutoScroll(true);
        mFormPanel.setCollapsible(true);
        mFormPanel.setWidth(400);
        mFormPanel.setLabelWidth(100);
        
        mFormPanel.setTitle("Dashboard Properties");
        mFormPanel.setDisabled(true);
        
        final TextField nameTextField = new TextField("Name", "name", 120);
        mFormPanel.add(nameTextField, new AnchorLayoutData("98%"));
        
        final Checkbox hiddenCheckBox = new Checkbox("Hidden", "hidden");
        mFormPanel.add(hiddenCheckBox, new AnchorLayoutData("98%"));
        
        final ComboBox catComboBox = new ComboBox("Category", "catId", 120);
        mFormPanel.add(catComboBox, new AnchorLayoutData("98%"));
        catComboBox.setStore(mCatStore);
        catComboBox.setValueField("id");
        catComboBox.setDisplayField("name");
        catComboBox.setMode(ComboBox.LOCAL);
        catComboBox.setForceSelection(true);
        catComboBox.setDisableKeyFilter(true);
        catComboBox.setLazyRender(true);
        catComboBox.setTriggerAction(ComboBox.ALL);
        
        final Button composeButton = new Button("Edit dashboard composition");
        mFormPanel.add(composeButton);
        composeButton.addListener(new ButtonListenerAdapter(){
			@Override
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("Dashboard composition");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(600);
				window.setHeight(400);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				window.add(new DashboardComposer(mAdminModule, (DashboardForm) mGridPanel.getSelectionModel().getSelected().getAsObject("object")));
				window.addButton(new Button("OK", new ButtonListenerAdapter(){

					@Override
					public void onClick(Button button, EventObject e) {
						window.close();
						window.destroy();
					}}));
				window.addButton(new Button("Cancel", new ButtonListenerAdapter(){

					@Override
					public void onClick(Button button, EventObject e) {
						final DashboardForm dashboard = (DashboardForm) mGridPanel.getSelectionModel().getSelected().getAsObject("object");
						getAdminModule().getAdminModuleService().getDashboard(dashboard.getId(), new AsyncCallback<DashboardForm>(){

							public void onFailure(Throwable caught)
                            {
								MessageBox.alert("Error", caught.getLocalizedMessage());
                            }

							public void onSuccess(DashboardForm result)
                            {
								dashboard.setColumns(result.getColumns());
								window.close();
								window.destroy();
                            }});
					}}));


				window.show();
			}});
        
        final ListParamPanel listParamPanel = new ListParamPanel(getAdminModule());
        mFormPanel.add(listParamPanel, new AnchorLayoutData("98% -100"));

        final Button commitButton = new Button("Save");
        mFormPanel.addButton(commitButton);
        
        final Button cancelButton = new Button("Cancel");
        mFormPanel.addButton(cancelButton);
        
        commitButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				final Record record = mGridPanel.getSelectionModel().getSelected();
				
				final AsyncCallback<DashboardForm> callback = new AsyncCallback<DashboardForm>(){

					public void onFailure(Throwable caught) {
						MessageBox.alert("Technical problem", "Unable to update dashboard");
					}

					public void onSuccess(DashboardForm result) {
						record.set("object", result);
						record.commit();
					}
				};
				
				mFormPanel.getForm().updateRecord(record);
				final DashboardForm dashboard = (DashboardForm) record.getAsObject("object");
				dashboard.setName(record.getAsString("name"));
				dashboard.setHidden(record.getAsBoolean("hidden"));
				ReportCategoryForm cat = (ReportCategoryForm) mCatStore.query("id", catComboBox.getValue())[0].getAsObject("object");
				record.set("catName", cat.getName());
				dashboard.setCategory(cat);
				dashboard.setParams(listParamPanel.getParameters());
				for (ReportParamForm param : listParamPanel.getParameters()) {
					param.setDashboard(dashboard);
				}
				getAdminModule().getAdminModuleService().updateDashboard(dashboard, callback);
			}});
        
        cancelButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				mFormPanel.getForm().loadRecord(mGridPanel.getSelectionModel().getSelected());
				listParamPanel.setParameters(((DashboardForm)mGridPanel.getSelectionModel().getSelected().getAsObject("object")).getParams());
			}});
        
        
		BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
        eastLayoutData.setMargins(new Margins(5, 0, 5, 5));
        eastLayoutData.setMinWidth(400);
        eastLayoutData.setSplit(true);
        
        mGridPanel.getSelectionModel().addListener(new RowSelectionListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter#onSelectionChange(com.gwtext.client.widgets.grid.RowSelectionModel)
			 */
			@Override
			public void onSelectionChange(RowSelectionModel sm) {
				if (sm.getSelected() != null)
				{
					mFormPanel.getForm().loadRecord(sm.getSelected());
					listParamPanel.setParameters(((DashboardForm)sm.getSelected().getAsObject("object")).getParams());
					mFormPanel.setDisabled(false);
				}
				else
				{
					mFormPanel.setDisabled(true);
				}
			}});
        
        add(mFormPanel, eastLayoutData);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
		final AsyncCallback<Collection<DashboardForm>> dashboardCallback = new AsyncCallback<Collection<DashboardForm>>() {
			public void onSuccess(Collection<DashboardForm> dashboards) {

				mStore.removeAll();
				mStore.commitChanges();
				for (DashboardForm dashboard : dashboards) {
					mStore.add(mRecordDef.createRecord(new Object[] { 
							dashboard,
							dashboard.getId(), 
							dashboard.getName(),
							dashboard.getHidden(),
							dashboard.getCategory() == null ? null : dashboard.getCategory().getName(),
							dashboard.getCategory() == null ? null : dashboard.getCategory().getId()}));
				}
				mStore.commitChanges();
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		final AsyncCallback<Collection<ReportCategoryForm>> catCallback = new AsyncCallback<Collection<ReportCategoryForm>>(){

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}

			public void onSuccess(Collection<ReportCategoryForm> result) {
				
				mCatStore.removeAll();
				for (ReportCategoryForm cat : result) {
					mCatStore.add(mCatRecordDef.createRecord(new Object[]{
							cat,
							cat.getId(), 
							cat.getName()}));
				}
				mCatStore.commitChanges();
				getAdminModule().getAdminModuleService().getDashboards(dashboardCallback);
			}};
		
			getAdminModule().getAdminModuleService().getReportCategories(catCallback);
	}
}
