/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.reports;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.paramchooser.ParamChooser;
import org.appspy.admin.client.reports.form.ReportParamChooserForm;
import org.appspy.admin.client.reports.form.ReportParamForm;
import org.appspy.admin.client.util.ParamChooserFactory;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.data.BooleanFieldDef;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.Checkbox;
import com.gwtext.client.widgets.form.ComboBox;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.Hidden;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.TriggerField;
import com.gwtext.client.widgets.form.event.ComboBoxListenerAdapter;
import com.gwtext.client.widgets.form.event.FieldListener;
import com.gwtext.client.widgets.form.event.FieldListenerAdapter;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.GridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtext.client.widgets.layout.RowLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListParamPanel extends Panel {
	
	protected AdminModule mAdminModule = null;
	
	protected GridPanel mGridPanel = null;
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"),
			new StringFieldDef("name"),
			new BooleanFieldDef("ask"),
			new StringFieldDef("prompt"),
			new StringFieldDef("default"),
			new IntegerFieldDef("chooserId"),
			new StringFieldDef("chooserName")
	});
	
	protected RecordDef mChooserRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"),
			new StringFieldDef("name")
	});
	
	protected Store mStore = null;
	protected Store mChooserStore = null;
	
	public ListParamPanel(AdminModule adminModule)
	{
		mAdminModule = adminModule;
		
		setLayout(new RowLayout());
		
		mStore = new Store(mRecordDef);
		mChooserStore = new Store(mChooserRecordDef);
		
		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig("NAME", "name", 100, true);
		ColumnConfig askColumnConfig = new ColumnConfig("ASK", "ask", 50, true);
		ColumnConfig promptColumnConfig = new ColumnConfig("PROMPT", "prompt", 100, true);
		ColumnConfig defaultColumnConfig = new ColumnConfig("DEFAULT", "default", 100, true);
		ColumnConfig chooserColumnConfig = new ColumnConfig("CHOOSER", "chooserName", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				nameColumnConfig,
				askColumnConfig,
				promptColumnConfig,
				defaultColumnConfig,
				chooserColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);

		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);
		mGridPanel.setStore(mStore);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(1);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		Toolbar toolbar = new Toolbar();
		ToolbarButton newButton = new ToolbarButton("Add", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New Parameter");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(450);
				window.setHeight(350);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);

				final ReportParamForm param = new ReportParamForm();
				final Record record = mRecordDef.createRecord(new Object[]{
						param,
						null, 
						"", 
						true,
						"",
						"",
						null,
						""});
				
				TextField nameTextField = new TextField("Name", "name");
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));
				
				Checkbox askCheckBox = new Checkbox("Ask", "ask");
				formPanel.add(askCheckBox, new AnchorLayoutData("100%"));
				
				TextField promptTextField = new TextField("Prompt", "prompt");
				formPanel.add(promptTextField, new AnchorLayoutData("100%"));
				
		        final ComboBox chooserComboBox = new ComboBox("Chooser", "chooserId");
		        formPanel.add(chooserComboBox, new AnchorLayoutData("100%"));
		        chooserComboBox.setStore(mChooserStore);
		        chooserComboBox.setValueField("id");
		        chooserComboBox.setDisplayField("name");
		        chooserComboBox.setMode(ComboBox.LOCAL);
		        chooserComboBox.setForceSelection(true);
		        chooserComboBox.setDisableKeyFilter(true);
		        chooserComboBox.setLazyRender(true);
		        chooserComboBox.setTriggerAction(ComboBox.ALL);

//		        TextField defaultField = new TextField("Default", "default");
//		        formPanel.add(defaultField, new AnchorLayoutData("100%"));
		        
		        final Hidden defaultField = new Hidden("default", "");
		        formPanel.add(defaultField);
		        
		        final TriggerField chooserField = new TriggerField(){

					@Override
					protected void onTriggerClick(EventObject event) {
						ReportParamChooserForm reportParamChooser = (ReportParamChooserForm) mChooserStore.query("id", chooserComboBox.getValue())[0].getAsObject("object");
						final Window chooserWindow = new Window();
						chooserWindow.setTitle("Choose default value");
						chooserWindow.setClosable(true);
						chooserWindow.setFrame(true);
						chooserWindow.setWidth(450);
						chooserWindow.setHeight(350);
						chooserWindow.setAutoScroll(true);
						chooserWindow.setPlain(true);
						chooserWindow.setLayout(new FitLayout());
						chooserWindow.setCloseAction(Window.CLOSE);
						chooserWindow.setModal(true);
						
						final TriggerField f = this;

						try
						{
							final ParamChooser paramChooser = new ParamChooserFactory().getParamChooser(reportParamChooser.getChooserAdminClass());
							paramChooser.setAdminModule(mAdminModule);
							paramChooser.setValue(defaultField.getValueAsString());
							chooserWindow.add(paramChooser.getChooserComponent());
							chooserWindow.addButton(new Button("OK", new ButtonListenerAdapter(){

								@Override
								public void onClick(Button button, EventObject e) {
									defaultField.setValue(paramChooser.getValue());
									paramChooser.updateTextField(f);
									chooserWindow.destroy();
								}}));
							chooserWindow.addButton(new Button("Cancel", new ButtonListenerAdapter(){

								@Override
								public void onClick(Button button, EventObject e) {
									chooserWindow.destroy();
								}}));
							
							chooserWindow.show();
						}
						catch (Exception ex)
						{
							GWT.log(ex.getMessage(), ex);
						}
					}};
				chooserField.setLabel("Default");
				chooserField.setReadOnly(true);
		        formPanel.add(chooserField, new AnchorLayoutData("100%"));
		        
		        chooserComboBox.addListener(new ComboBoxListenerAdapter(){
		        	
					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.form.event.ComboBoxListenerAdapter#doBeforeSelect(com.gwtext.client.widgets.form.ComboBox, com.gwtext.client.data.Record, int)
					 */
					@Override
					public boolean doBeforeSelect(ComboBox comboBox, Record record, int index) {
						try
						{
							if (!comboBox.getValue().equals(record.getAsString("id")))
							{
								defaultField.setValue(null);
								chooserField.setValue(null);
							}
						}
						catch (Exception ex)
						{
						}
						return true;
		        	}});
		        
		        final Button createButton = new Button("Create");
				formPanel.addButton(createButton);
				
				formPanel.getForm().loadRecord(record);

				createButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						param.setName(record.getAsString("name"));
						param.setAsk(record.getAsBoolean("ask"));
						param.setPrompt(record.getAsString("prompt"));
						param.setDefault(record.getAsString("default"));
						ReportParamChooserForm chooser = (ReportParamChooserForm) mChooserStore.query("id", chooserComboBox.getValueAsString())[0].getAsObject("object");
						record.set("chooserName", chooser.getName());
						param.setChooser(chooser);
						
						mStore.add(record);
						record.commit();
						window.close();
					}});
				
				window.add(formPanel);
				window.show();
			}
		});
		newButton.setIcon("images/silk/add.gif");
		toolbar.addButton(newButton);

		toolbar.addSeparator();

		ToolbarButton editButton = new ToolbarButton("Edit", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("Edit Parameter");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(450);
				window.setHeight(350);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);

				final Record record = mGridPanel.getSelectionModel().getSelected();
				final ReportParamForm param = (ReportParamForm) record.getAsObject("object");
				
				final FieldListener fieldListener = new FieldListenerAdapter(){
					@Override
					public void onRender(Component component) {
						formPanel.getForm().loadRecord(record);
					}};
				TextField nameTextField = new TextField("Name", "name");
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));
				nameTextField.addListener(fieldListener);
				
				Checkbox askCheckBox = new Checkbox("Ask", "ask");
				formPanel.add(askCheckBox, new AnchorLayoutData("100%"));
				askCheckBox.addListener(fieldListener);
				
				TextField promptTextField = new TextField("Prompt", "prompt");
				formPanel.add(promptTextField, new AnchorLayoutData("100%"));
				promptTextField.addListener(fieldListener);
				
		        final ComboBox chooserComboBox = new ComboBox("Chooser", "chooserId");
		        formPanel.add(chooserComboBox, new AnchorLayoutData("100%"));
		        chooserComboBox.setStore(mChooserStore);
		        chooserComboBox.setValueField("id");
		        chooserComboBox.setDisplayField("name");
		        chooserComboBox.setMode(ComboBox.LOCAL);
		        chooserComboBox.setForceSelection(true);
		        chooserComboBox.setDisableKeyFilter(true);
		        chooserComboBox.setLazyRender(true);
		        chooserComboBox.setTriggerAction(ComboBox.ALL);
		        chooserComboBox.addListener(fieldListener);

//				TextField defaultTextField = new TextField("Default", "default");
//				formPanel.add(defaultTextField, new AnchorLayoutData("100%"));
//				defaultTextField.addListener(fieldListener);

		        final Hidden defaultField = new Hidden("default", "default");
		        formPanel.add(defaultField);
		        defaultField.addListener(fieldListener);
		        
		        final TriggerField chooserField = new TriggerField(){

					@Override
					protected void onTriggerClick(EventObject event) {
						ReportParamChooserForm reportParamChooser = (ReportParamChooserForm) mChooserStore.query("id", chooserComboBox.getValue())[0].getAsObject("object");
						final Window chooserWindow = new Window();
						chooserWindow.setTitle("Choose default value");
						chooserWindow.setClosable(true);
						chooserWindow.setFrame(true);
						chooserWindow.setWidth(450);
						chooserWindow.setHeight(350);
						chooserWindow.setAutoScroll(true);
						chooserWindow.setPlain(true);
						chooserWindow.setLayout(new FitLayout());
						chooserWindow.setCloseAction(Window.CLOSE);
						chooserWindow.setModal(true);
						
						final TriggerField f = this;

						try
						{
							final ParamChooser paramChooser = new ParamChooserFactory().getParamChooser(reportParamChooser.getChooserAdminClass());
							paramChooser.setAdminModule(mAdminModule);
							paramChooser.setValue(defaultField.getValueAsString());
							chooserWindow.add(paramChooser.getChooserComponent());
							chooserWindow.addButton(new Button("OK", new ButtonListenerAdapter(){

								@Override
								public void onClick(Button button, EventObject e) {
									defaultField.setValue(paramChooser.getValue());
									paramChooser.updateTextField(f);
									chooserWindow.destroy();
								}}));
							chooserWindow.addButton(new Button("Cancel", new ButtonListenerAdapter(){

								@Override
								public void onClick(Button button, EventObject e) {
									chooserWindow.destroy();
								}}));
							
							chooserWindow.show();
						}
						catch (Exception ex)
						{
							GWT.log(ex.getMessage(), ex);
						}
					}};
				chooserField.setLabel("Default");
				chooserField.setReadOnly(true);
		        formPanel.add(chooserField, new AnchorLayoutData("100%"));
		        
		        try
		        {
		        	ReportParamChooserForm reportParamChooser = ((ReportParamForm) record.getAsObject("object")).getChooser();
		        	final ParamChooser paramChooser = new ParamChooserFactory().getParamChooser(reportParamChooser.getChooserAdminClass());
		        	paramChooser.setAdminModule(mAdminModule);
		        	paramChooser.setValue(record.getAsString("default"));
		        	paramChooser.updateTextField(chooserField);
		        }
		        catch (Exception ex)
		        {
		        	GWT.log(ex.getMessage(), ex);
		        }
		        
		        chooserComboBox.addListener(new ComboBoxListenerAdapter(){
		        	
					@Override
					public void onSelect(ComboBox comboBox, Record record, int index) {
					}

					@Override
					public boolean doBeforeSelect(ComboBox comboBox, Record comboRecord, int index) {
						try
						{
							if (!comboBox.getValue().equals(comboRecord.getAsString("id")))
							{
								defaultField.setValue(null);
								chooserField.setValue(null);
							}
						}
						catch (Exception ex)
						{
							GWT.log(ex.getMessage(), ex);
						}
						return true;
		        	}});

		        final Button saveButton = new Button("Save");
				formPanel.addButton(saveButton);
				
								
				saveButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						param.setName(record.getAsString("name"));
						param.setAsk(record.getAsBoolean("ask"));
						param.setPrompt(record.getAsString("prompt"));
						param.setDefault(record.getAsString("default"));
						ReportParamChooserForm chooser = (ReportParamChooserForm) mChooserStore.query("id", chooserComboBox.getValueAsString())[0].getAsObject("object");
						record.set("chooserName", chooser.getName());
						param.setChooser(chooser);
						
						record.commit();
						window.close();
					}});
				
				final Button cancelButton = new Button("Cancel");
				formPanel.addButton(cancelButton);
				
				cancelButton.addListener(new ButtonListenerAdapter(){
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().loadRecord(record);
				        try
				        {
				        	ReportParamChooserForm reportParamChooser = ((ReportParamForm) record.getAsObject("object")).getChooser();
				        	final ParamChooser paramChooser = new ParamChooserFactory().getParamChooser(reportParamChooser.getChooserAdminClass());
				        	paramChooser.setAdminModule(mAdminModule);
				        	paramChooser.setValue(record.getAsString("default"));
				        	paramChooser.updateTextField(chooserField);
				        }
				        catch (Exception ex)
				        {
				        	GWT.log(ex.getMessage(), ex);
				        }
					}});

				window.add(formPanel);
				window.show();
			}
		});
		editButton.setIcon("images/silk/add.gif");
		toolbar.addButton(editButton);

		toolbar.addSeparator();

		ToolbarButton delButton = new ToolbarButton("Del", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				try {
					final Record record = mGridPanel.getSelectionModel().getSelected();
					if (record != null)
					{
						mStore.remove(record);
					}
				} catch (Exception ex) {
					MessageBox.alert(ex.getMessage());
				}
			}
		});
		delButton.setIcon("images/silk/delete.gif");
		toolbar.addButton(delButton);
		
		toolbar.addSeparator();
		toolbar.addSpacer();
		
		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		
//		add(new Label("Parameters"));
		
		mGridPanel.setTitle("Parameters");
		add(mGridPanel);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
		AsyncCallback<Collection<ReportParamChooserForm>> callback = new AsyncCallback<Collection<ReportParamChooserForm>>() {
			public void onSuccess(Collection<ReportParamChooserForm> choosers) {
				
				try
				{

				mChooserStore.removeAll();
				mChooserStore.commitChanges();
				for (ReportParamChooserForm chooser : choosers) {
					mChooserStore.add(mChooserRecordDef.createRecord(new Object[] {
							chooser,
							chooser.getId(), 
							chooser.getName() }));
				}
				mChooserStore.commitChanges();
				
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		try
		{
			
			mAdminModule.getAdminModuleService().getReportParamChoosers(callback);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public Set<ReportParamForm> getParameters()
	{
		HashSet<ReportParamForm> params = new HashSet<ReportParamForm>();
		
		for (Record record : mStore.getRecords()) {
			params.add((ReportParamForm) record.getAsObject("object"));
		}
		
		return params;
	}
	
	public void setParameters(Set<ReportParamForm> params)
	{
		mStore.removeAll();
		
		for (ReportParamForm param : params) {
			mStore.add(mRecordDef.createRecord(new Object[]{
					param,
					param.getId(),
					param.getName(),
					param.getAsk(),
					param.getPrompt(),
					param.getDefault(),
					param.getChooser() == null ? null : param.getChooser().getId(),
					param.getChooser() == null ? "" : param.getChooser().getName()
			}));
		}
	}
}
