/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.reports;

import java.util.Collection;

import org.appspy.admin.client.AbstractPanel;
import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.reports.form.ReportCategoryForm;
import org.appspy.admin.client.reports.form.ReportForm;
import org.appspy.admin.client.reports.form.ReportParamForm;
import org.appspy.admin.client.reports.form.ReportingSystemForm;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.NamedFrame;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.Widget;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.data.BooleanFieldDef;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.data.XmlReader;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.Checkbox;
import com.gwtext.client.widgets.form.ComboBox;
import com.gwtext.client.widgets.form.Form;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.Hidden;
import com.gwtext.client.widgets.form.Label;
import com.gwtext.client.widgets.form.MultiFieldPanel;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.event.FormListenerAdapter;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.ColumnLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtextux.client.widgets.grid.plugins.GridSearchPlugin;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListReportPanel extends AbstractPanel {

	public static final String TAB_ID = "ListReportPanel";
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[] { 
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name"), 
			new BooleanFieldDef("hidden"),
			new StringFieldDef("sysName"),
			new IntegerFieldDef("sysId"),
			new StringFieldDef("catName"),
			new IntegerFieldDef("catId")});
	
	protected RecordDef mCatRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name")});
	
	protected RecordDef mRepSysRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name")});
	
	protected EditorGridPanel mGridPanel = null;
	protected FormPanel mFormPanel = null;
	
	protected Store mStore = null;
	protected Store mCatStore = null;
	protected Store mRepSysStore = null;
	
	public ListReportPanel(AdminModule adminModule) {
		
		super(adminModule);

		setTitle("Reports");

		setLayout(new BorderLayout());
		setAutoScroll(true);

		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig("NAME", "name", 100, true);
		ColumnConfig hiddenColumnConfig = new ColumnConfig("HIDDEN", "hidden", 50, true);
		ColumnConfig catNameColumnConfig = new ColumnConfig("CATEGORY", "catName", 100, true);
		ColumnConfig sysNameColumnConfig = new ColumnConfig("SYSTEM", "sysName", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				nameColumnConfig,
				hiddenColumnConfig,
				catNameColumnConfig,
				sysNameColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);

		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(1);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		mStore = new Store(mRecordDef);
		mGridPanel.setStore(mStore);
		
		mCatStore = new Store(mCatRecordDef);
		mRepSysStore = new Store(mRepSysRecordDef);
		
		Toolbar toolbar = new Toolbar();
		ToolbarButton newButton = new ToolbarButton("Add", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New report");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(600);
				window.setHeight(400);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);
				
				final ReportForm report = new ReportForm();
				final Record record = mRecordDef.createRecord(new Object[]{
						report,
						null, 
						"",
						false,
						"",
						null,
						"",
						null});
				
				TextField nameTextField = new TextField("Name", "name");
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));
				
				Checkbox hiddenCheckBox = new Checkbox("Hidden", "hidden");
				formPanel.add(hiddenCheckBox, new AnchorLayoutData("100%"));

		        final ComboBox catComboBox = new ComboBox("Category", "catId");
		        formPanel.add(catComboBox, new AnchorLayoutData("100%"));
		        catComboBox.setStore(mCatStore);
		        catComboBox.setValueField("id");
		        catComboBox.setDisplayField("name");
		        catComboBox.setMode(ComboBox.LOCAL);
		        catComboBox.setForceSelection(true);
		        catComboBox.setDisableKeyFilter(true);
		        catComboBox.setLazyRender(true);
		        catComboBox.setTriggerAction(ComboBox.ALL);

		        final ComboBox sysComboBox = new ComboBox("Rep. System", "sysId");
		        formPanel.add(sysComboBox, new AnchorLayoutData("100%"));
		        sysComboBox.setStore(mRepSysStore);
		        sysComboBox.setValueField("id");
		        sysComboBox.setDisplayField("name");
		        sysComboBox.setMode(ComboBox.LOCAL);
		        sysComboBox.setForceSelection(true);
		        sysComboBox.setDisableKeyFilter(true);
		        sysComboBox.setLazyRender(true);
		        sysComboBox.setTriggerAction(ComboBox.ALL);
		        
		        final TextField fileField = new TextField("Report File");
		        fileField.setInputType("file");
		        fileField.setName("reportFile");
		        formPanel.add(fileField, new AnchorLayoutData("100%"));
		        formPanel.setUrl("uploadReportServlet");
		        formPanel.setFileUpload(true);
		        
		        final ListParamPanel listParamPanel = new ListParamPanel(getAdminModule());
		        formPanel.add(listParamPanel, new AnchorLayoutData("100% -100"));
		        formPanel.setHeight(200);

		        final Hidden descReportIdField = new Hidden("reportId", "");
		        formPanel.add(descReportIdField);
		        
		        final XmlReader reader = new XmlReader("data", new RecordDef(new
		        		FieldDef[] { new StringFieldDef("responseMsg") }));
		        		        reader.setSuccess("@success");

		        final XmlReader errorReader = new XmlReader("data", new
		        		RecordDef(new FieldDef[] { new StringFieldDef("responseMsg") }));
		        		        errorReader.setSuccess("@success");
		        
		        formPanel.setReader(reader);
		        formPanel.setErrorReader(errorReader);

		        final Button createButton = new Button("Create");
				formPanel.addButton(createButton);
				
				formPanel.getForm().loadRecord(record);

				createButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						report.setName(record.getAsString("name"));
						report.setHidden(record.getAsBoolean("hidden"));
						ReportingSystemForm sys = (ReportingSystemForm) mRepSysStore.query("id", sysComboBox.getValueAsString())[0].getAsObject("object");
						record.set("sysName", sys.getName());
						report.setReportingSystem(sys);
						ReportCategoryForm cat = (ReportCategoryForm) mCatStore.query("id", catComboBox.getValueAsString())[0].getAsObject("object");
						record.set("catName", cat.getName());
						report.setCategory(cat);
						report.setParams(listParamPanel.getParameters());
						for (ReportParamForm param : listParamPanel.getParameters()) {
							param.setReport(report);
						}
						final AsyncCallback<ReportForm> callback = new AsyncCallback<ReportForm>(){

							public void onFailure(Throwable caught) {
								MessageBox.alert("Technical problem", "Unable to create report");
							}

							public void onSuccess(ReportForm result) {
								record.set("object", result);
								record.set("id", result.getId());
								mStore.add(record);
								record.commit();
								
								Form form = formPanel.getForm();
								form.addListener(new FormListenerAdapter(){

									/* (non-Javadoc)
									 * @see com.gwtext.client.widgets.form.event.FormListenerAdapter#onActionComplete(com.gwtext.client.widgets.form.Form, int, java.lang.String)
									 */
									@Override
									public void onActionComplete(Form form, int httpStatus, String responseText) {
//										MessageBox.alert("Result", responseText); 
										window.close();
									}

									/* (non-Javadoc)
									 * @see com.gwtext.client.widgets.form.event.FormListenerAdapter#onActionFailed(com.gwtext.client.widgets.form.Form, int, java.lang.String)
									 */
									@Override
									public void onActionFailed(Form form, int httpStatus, String responseText) {
										MessageBox.alert("Problem during file upload.");
									}});
								descReportIdField.setValue(result.getId().toString());
								form.submit("uploadReportServlet");
								
//								window.close();
								
							}
						};
						
						getAdminModule().getAdminModuleService().saveReport(report, callback);
					}});
				
				window.add(formPanel);
				window.show();
			}
		});
		newButton.setIcon("images/silk/add.gif");
		toolbar.addButton(newButton);

		toolbar.addSeparator();

		ToolbarButton delButton = new ToolbarButton("Del", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				try {
					final Record record = mGridPanel.getSelectionModel().getSelected();
					if (record != null)
					{
						MessageBox.confirm("Confirm", "Are you sure you want to delete this report ?",
								new MessageBox.ConfirmCallback() {

									public void execute(String btnID) {
										if (btnID.equals("yes")) {
											ReportForm reportDescription = (ReportForm) record.getAsObject("object");
											getAdminModule().getAdminModuleService().deleteReport(reportDescription, new AsyncCallback<ReportForm>(){

												public void onFailure(Throwable caught) {
													MessageBox.alert("Unable to delete report", caught.getLocalizedMessage());
												}

												public void onSuccess(ReportForm result) {
													mStore.remove(record);
												}});
										}
									}
								});
					}
				} catch (Exception ex) {
					MessageBox.alert(ex.getMessage());
				}
			}
		});
		delButton.setIcon("images/silk/delete.gif");
		toolbar.addButton(delButton);
		
		toolbar.addSeparator();
		toolbar.addSpacer();
		
		ToolbarButton refreshButton = new ToolbarButton("Refresh", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				refreshDatas();
			}});
		refreshButton.setIcon("images/icons/fam/table_refresh.png");
		toolbar.addButton(refreshButton);

		ToolbarButton exportButton = new ToolbarButton("Export", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				ExportWindow window = new ExportWindow(getAdminModule());
				
				window.show();
				
			}});
		toolbar.addButton(exportButton);

		ToolbarButton importButton = new ToolbarButton("Import", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				ImportWindow window = new ImportWindow(getAdminModule());
				
				window.show();
				
			}});
		toolbar.addButton(importButton);

		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		GridSearchPlugin gridSearch = new GridSearchPlugin(GridSearchPlugin.TOP);  
		gridSearch.setMode(GridSearchPlugin.LOCAL);  
		mGridPanel.addPlugin(gridSearch);  

        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 5, 0, 5));
        centerLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        centerLayoutData.setMinSize(200);
        centerLayoutData.setMinWidth(200);
        
        add(mGridPanel, centerLayoutData);

        mFormPanel = new FormPanel();
        mFormPanel.setPaddings(10, 10, 10, 10);
        mFormPanel.setFrame(true);
        mFormPanel.setAutoScroll(true);
        mFormPanel.setCollapsible(true);
        mFormPanel.setWidth(400);
        mFormPanel.setLabelWidth(100);
        
        mFormPanel.setTitle("Report Properties");
        mFormPanel.setDisabled(true);
        
        final TextField nameTextField = new TextField("Name", "name", 120);
        mFormPanel.add(nameTextField, new AnchorLayoutData("98%"));
        
        final Checkbox hiddenCheckBox = new Checkbox("Hidden", "hidden");
        mFormPanel.add(hiddenCheckBox, new AnchorLayoutData("98%"));
        
        final ComboBox catComboBox = new ComboBox("Category", "catId", 120);
        mFormPanel.add(catComboBox, new AnchorLayoutData("98%"));
        catComboBox.setStore(mCatStore);
        catComboBox.setValueField("id");
        catComboBox.setDisplayField("name");
        catComboBox.setMode(ComboBox.LOCAL);
        catComboBox.setForceSelection(true);
        catComboBox.setDisableKeyFilter(true);
        catComboBox.setLazyRender(true);
        catComboBox.setTriggerAction(ComboBox.ALL);
        
        final ComboBox sysComboBox = new ComboBox("Rep. System", "sysId", 120);
        mFormPanel.add(sysComboBox, new AnchorLayoutData("98%"));
        sysComboBox.setStore(mRepSysStore);
        sysComboBox.setValueField("id");
        sysComboBox.setDisplayField("name");
        sysComboBox.setMode(ComboBox.LOCAL);
        sysComboBox.setForceSelection(true);
        sysComboBox.setDisableKeyFilter(true);
        sysComboBox.setLazyRender(true);
        sysComboBox.setTriggerAction(ComboBox.ALL);
        
        MultiFieldPanel reportFile = new MultiFieldPanel();
        mFormPanel.add(reportFile, new AnchorLayoutData("98%"));
        
        final Label reportFileLabel = new Label("Rep. File:");
        reportFile.addToRow(reportFileLabel, 100 + 5);
        
        final Button uploadButton = new Button("Upload");
        reportFile.addToRow(uploadButton, new ColumnLayoutData(0.5));
        uploadButton.addListener(new ButtonListenerAdapter(){
			@Override
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New report");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(450);
				window.setHeight(350);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);

		        final TextField fileField = new TextField("Report File");
		        fileField.setInputType("file");
		        fileField.setName("reportFile");
		        formPanel.add(fileField, new AnchorLayoutData("98%"));
		        formPanel.setUrl("uploadReportServlet");
		        formPanel.setFileUpload(true);
		        
		        final Hidden descReportIdField = new Hidden("reportId", "");
		        formPanel.add(descReportIdField);
		        descReportIdField.setValue(((ReportForm)mGridPanel.getSelectionModel().getSelected().getAsObject("object")).getId().toString());
		        
		        final XmlReader reader = new XmlReader("data", new RecordDef(new
		        		FieldDef[] { new StringFieldDef("responseMsg") }));
		        		        reader.setSuccess("@success");

		        final XmlReader errorReader = new XmlReader("data", new
		        		RecordDef(new FieldDef[] { new StringFieldDef("responseMsg") }));
		        		        errorReader.setSuccess("@success");
		        
		        formPanel.setReader(reader);
		        formPanel.setErrorReader(errorReader);

				Form form = formPanel.getForm();
				form.addListener(new FormListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.form.event.FormListenerAdapter#onActionComplete(com.gwtext.client.widgets.form.Form, int, java.lang.String)
					 */
					@Override
					public void onActionComplete(Form form, int httpStatus, String responseText) {
						window.close();
					}

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.form.event.FormListenerAdapter#onActionFailed(com.gwtext.client.widgets.form.Form, int, java.lang.String)
					 */
					@Override
					public void onActionFailed(Form form, int httpStatus, String responseText) {
						MessageBox.alert("Problem during file upload.");
					}});
				
				Button uploadButton = new Button("Upload");
				formPanel.addButton(uploadButton);
				uploadButton.addListener(new ButtonListenerAdapter(){

					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().submit("uploadReportServlet");
					}});
				
				window.add(formPanel);
				window.show();
			}});
        
        final Button downloadButton = new Button("Download");
        reportFile.addToRow(downloadButton, new ColumnLayoutData(0.5));
        downloadButton.addListener(new ButtonListenerAdapter(){
			@Override
			public void onClick(Button button, EventObject e) {
				
		        boolean frameExists = (RootPanel.get("downloadiframe") != null);
		        if(frameExists) {
		            Widget widgetFrame = (Widget)RootPanel.get("downloadiframe");
		            widgetFrame.removeFromParent();
		        }
		        
		        String frameUrl = GWT.getModuleBaseURL() + "downloadReportServlet?reportId=" + ((ReportForm)mGridPanel.getSelectionModel().getSelected().getAsObject("object")).getId().toString();
		        NamedFrame frame = new NamedFrame("downloadiframe");
		        frame.setUrl(frameUrl);
		        frame.setVisible(false);
		        
		        RootPanel.get().add(frame);
			}});
        
        final ListParamPanel listParamPanel = new ListParamPanel(getAdminModule());
        mFormPanel.add(listParamPanel, new AnchorLayoutData("98% -100"));

        final Button commitButton = new Button("Save");
        mFormPanel.addButton(commitButton);
        
        final Button cancelButton = new Button("Cancel");
        mFormPanel.addButton(cancelButton);
        
        commitButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				final Record record = mGridPanel.getSelectionModel().getSelected();
				
				final AsyncCallback<ReportForm> callback = new AsyncCallback<ReportForm>(){

					public void onFailure(Throwable caught) {
						MessageBox.alert("Technical problem", "Unable to update report description");
					}

					public void onSuccess(ReportForm result) {
						record.set("object", result);
						record.commit();
					}
				};
				
				mFormPanel.getForm().updateRecord(record);
				final ReportForm report = (ReportForm) record.getAsObject("object");
				report.setName(record.getAsString("name"));
				report.setHidden(record.getAsBoolean("hidden"));
				ReportingSystemForm sys = (ReportingSystemForm) mRepSysStore.query("id", sysComboBox.getValue())[0].getAsObject("object");
				record.set("sysName", sys.getName());
				report.setReportingSystem(sys);
				ReportCategoryForm cat = (ReportCategoryForm) mCatStore.query("id", catComboBox.getValue())[0].getAsObject("object");
				record.set("catName", cat.getName());
				report.setCategory(cat);
				report.setParams(listParamPanel.getParameters());
				for (ReportParamForm param : listParamPanel.getParameters()) {
					param.setReport(report);
				}
				getAdminModule().getAdminModuleService().updateReport(report, callback);
			}});
        
        cancelButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				mFormPanel.getForm().loadRecord(mGridPanel.getSelectionModel().getSelected());
				listParamPanel.setParameters(((ReportForm)mGridPanel.getSelectionModel().getSelected().getAsObject("object")).getParams());
			}});
        
        
		BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
        eastLayoutData.setMargins(new Margins(5, 0, 5, 5));
        eastLayoutData.setMinWidth(400);
        eastLayoutData.setSplit(true);
        
        mGridPanel.getSelectionModel().addListener(new RowSelectionListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter#onSelectionChange(com.gwtext.client.widgets.grid.RowSelectionModel)
			 */
			@Override
			public void onSelectionChange(RowSelectionModel sm) {
				if (sm.getSelected() != null)
				{
					mFormPanel.getForm().loadRecord(sm.getSelected());
					listParamPanel.setParameters(((ReportForm)sm.getSelected().getAsObject("object")).getParams());
					mFormPanel.setDisabled(false);
				}
				else
				{
					mFormPanel.setDisabled(true);
				}
			}});
        
        add(mFormPanel, eastLayoutData);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
		final AsyncCallback<Collection<ReportForm>> reportCallback = new AsyncCallback<Collection<ReportForm>>() {
			public void onSuccess(Collection<ReportForm> reports) {

				mStore.removeAll();
				mStore.commitChanges();
				for (ReportForm report : reports) {
					mStore.add(mRecordDef.createRecord(new Object[] { 
							report,
							report.getId(), 
							report.getName(),
							report.getHidden(),
							report.getReportingSystem() == null ? null : report.getReportingSystem().getName(),
							report.getReportingSystem() == null ? null : report.getReportingSystem().getId(),
							report.getCategory() == null ? null : report.getCategory().getName(),
							report.getCategory() == null ? null : report.getCategory().getId()}));
				}
				mStore.commitChanges();
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		final AsyncCallback<Collection<ReportCategoryForm>> catCallback = new AsyncCallback<Collection<ReportCategoryForm>>(){

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}

			public void onSuccess(Collection<ReportCategoryForm> result) {
				
				mCatStore.removeAll();
				for (ReportCategoryForm cat : result) {
					mCatStore.add(mCatRecordDef.createRecord(new Object[]{
							cat,
							cat.getId(), 
							cat.getName()}));
				}
				mCatStore.commitChanges();
				getAdminModule().getAdminModuleService().getReports(reportCallback);
			}};
		
			final AsyncCallback<Collection<ReportingSystemForm>> sysCallback = new AsyncCallback<Collection<ReportingSystemForm>>(){

				public void onFailure(Throwable caught) {
					MessageBox.alert("Error", caught.getLocalizedMessage());
				}

				public void onSuccess(Collection<ReportingSystemForm> result) {
					
					mRepSysStore.removeAll();
					for (ReportingSystemForm sys : result) {
						mRepSysStore.add(mRepSysRecordDef.createRecord(new Object[]{
								sys,
								sys.getId(), 
								sys.getName()}));
					}
					mRepSysStore.commitChanges();
					getAdminModule().getAdminModuleService().getReportCategories(catCallback);
				}};
			
			getAdminModule().getAdminModuleService().getReportingSystems(sysCallback);
	}
}
