/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.scheduling;

import java.util.Collection;

import org.appspy.admin.client.AbstractPanel;
import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.scheduling.form.JobDeclarationForm;
import org.appspy.admin.client.scheduling.form.JobDeclarationParamForm;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtextux.client.widgets.grid.plugins.GridSearchPlugin;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListJobDeclarationPanel extends AbstractPanel {

	public static final String TAB_ID = "ListJobDeclarationPanel";
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[] { 
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name"), 
			new StringFieldDef("className")});
	
	protected EditorGridPanel mGridPanel = null;
	protected FormPanel mFormPanel = null;
	
	protected Store mStore = null;
	
	public ListJobDeclarationPanel(AdminModule adminModule) {
		
		super(adminModule);

		setTitle("Job Declarations");

		setLayout(new BorderLayout());
		setAutoScroll(true);

		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig("NAME", "name", 100, true);
		ColumnConfig classNameColumnConfig = new ColumnConfig("CLASS NAME", "className", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				nameColumnConfig,
				classNameColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);

		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(1);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		mStore = new Store(mRecordDef);
		mGridPanel.setStore(mStore);
		
		Toolbar toolbar = new Toolbar();
		ToolbarButton newButton = new ToolbarButton("Add", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New job declaration");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(600);
				window.setHeight(400);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);
				
				final JobDeclarationForm jobDeclaration = new JobDeclarationForm();
				final Record record = mRecordDef.createRecord(new Object[]{
						jobDeclaration,
						null, 
						"", 
						""});
				
				TextField nameTextField = new TextField("Name", "name");
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));

				TextField classNameTextField = new TextField("Class Name", "className");
				formPanel.add(classNameTextField, new AnchorLayoutData("100%"));

		        final ListJobDeclarationParamPanel listParamPanel = new ListJobDeclarationParamPanel(getAdminModule());
		        formPanel.add(listParamPanel, new AnchorLayoutData("100% -100"));
		        formPanel.setHeight(200);

		        final Button createButton = new Button("Create");
				formPanel.addButton(createButton);
				
				formPanel.getForm().loadRecord(record);

				createButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						jobDeclaration.setName(record.getAsString("name"));
						jobDeclaration.setClassName(record.getAsString("className"));
						jobDeclaration.setParams(listParamPanel.getParameters());
						for (JobDeclarationParamForm param : listParamPanel.getParameters()) {
							param.setJobDeclaration(jobDeclaration);
						}
						final AsyncCallback<JobDeclarationForm> callback = new AsyncCallback<JobDeclarationForm>(){

							public void onFailure(Throwable caught) {
								MessageBox.alert("Unable to create job declaration", caught.getMessage());
							}

							public void onSuccess(JobDeclarationForm result) {
								record.set("object", result);
								record.set("id", result.getId());
								mStore.add(record);
								record.commit();
								
								window.close();
							}
						};
						
						getAdminModule().getAdminModuleService().saveJobDeclaration(jobDeclaration, callback);
					}});
				
				window.add(formPanel);
				window.show();
			}
		});
		newButton.setIcon("images/silk/add.gif");
		toolbar.addButton(newButton);

		toolbar.addSeparator();

		ToolbarButton delButton = new ToolbarButton("Del", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				try {
					final Record record = mGridPanel.getSelectionModel().getSelected();
					if (record != null)
					{
						MessageBox.confirm("Confirm", "Are you sure you want to delete this job declaration ?",
								new MessageBox.ConfirmCallback() {

									public void execute(String btnID) {
										if (btnID.equals("yes")) {
											JobDeclarationForm jobDeclaration = (JobDeclarationForm) record.getAsObject("object");
											getAdminModule().getAdminModuleService().deleteJobDeclaration(jobDeclaration, new AsyncCallback<JobDeclarationForm>(){

												public void onFailure(Throwable caught) {
													MessageBox.alert("Unable to delete job declaration", caught.getLocalizedMessage());
												}

												public void onSuccess(JobDeclarationForm result) {
													mStore.remove(record);
												}});
										}
									}
								});
					}
				} catch (Exception ex) {
					MessageBox.alert(ex.getMessage());
				}
			}
		});
		delButton.setIcon("images/silk/delete.gif");
		toolbar.addButton(delButton);
		
		toolbar.addSeparator();
		toolbar.addSpacer();
		
		ToolbarButton refreshButton = new ToolbarButton("Refresh", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				refreshDatas();
			}});
		refreshButton.setIcon("images/icons/fam/table_refresh.png");
		toolbar.addButton(refreshButton);

		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		GridSearchPlugin gridSearch = new GridSearchPlugin(GridSearchPlugin.TOP);  
		gridSearch.setMode(GridSearchPlugin.LOCAL);  
		mGridPanel.addPlugin(gridSearch);  

        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 5, 0, 5));
        centerLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        centerLayoutData.setMinSize(200);
        centerLayoutData.setMinWidth(200);
        
        add(mGridPanel, centerLayoutData);

        mFormPanel = new FormPanel();
        mFormPanel.setPaddings(10, 10, 10, 10);
        mFormPanel.setFrame(true);
        mFormPanel.setAutoScroll(true);
        mFormPanel.setCollapsible(true);
        mFormPanel.setWidth(400);
        mFormPanel.setLabelWidth(100);
        
        mFormPanel.setTitle("Job Declaration Properties");
        mFormPanel.setDisabled(true);
        
        final TextField nameTextField = new TextField("Name", "name", 120);
        mFormPanel.add(nameTextField, new AnchorLayoutData("98%"));
        
        final TextField classNameTextField = new TextField("Class Name", "className", 120);
        mFormPanel.add(classNameTextField, new AnchorLayoutData("98%"));
        
        final ListJobDeclarationParamPanel listParamPanel = new ListJobDeclarationParamPanel(getAdminModule());
        mFormPanel.add(listParamPanel, new AnchorLayoutData("98% -100"));

        final Button commitButton = new Button("Save");
        mFormPanel.addButton(commitButton);
        
        final Button cancelButton = new Button("Cancel");
        mFormPanel.addButton(cancelButton);
        
        commitButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				final Record record = mGridPanel.getSelectionModel().getSelected();
				
				final AsyncCallback<JobDeclarationForm> callback = new AsyncCallback<JobDeclarationForm>(){

					public void onFailure(Throwable caught) {
						MessageBox.alert("Unable to update job declaration", caught.getMessage());
					}

					public void onSuccess(JobDeclarationForm result) {
						record.set("object", result);
						record.commit();
					}
				};
				
				mFormPanel.getForm().updateRecord(record);
				final JobDeclarationForm jobDeclaration = (JobDeclarationForm) record.getAsObject("object");
				jobDeclaration.setName(record.getAsString("name"));
				jobDeclaration.setName(record.getAsString("className"));
				jobDeclaration.setParams(listParamPanel.getParameters());
				for (JobDeclarationParamForm param : listParamPanel.getParameters()) {
					param.setJobDeclaration(jobDeclaration);
				}
				getAdminModule().getAdminModuleService().updateJobDeclaration(jobDeclaration, callback);
			}});
        
        cancelButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				mFormPanel.getForm().loadRecord(mGridPanel.getSelectionModel().getSelected());
				listParamPanel.setParameters(((JobDeclarationForm)mGridPanel.getSelectionModel().getSelected().getAsObject("object")).getParams());
			}});
        
        
		BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
        eastLayoutData.setMargins(new Margins(5, 0, 5, 5));
        eastLayoutData.setMinWidth(400);
        eastLayoutData.setSplit(true);
        
        mGridPanel.getSelectionModel().addListener(new RowSelectionListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter#onSelectionChange(com.gwtext.client.widgets.grid.RowSelectionModel)
			 */
			@Override
			public void onSelectionChange(RowSelectionModel sm) {
				if (sm.getSelected() != null)
				{
					mFormPanel.getForm().loadRecord(sm.getSelected());
					listParamPanel.setParameters(((JobDeclarationForm)sm.getSelected().getAsObject("object")).getParams());
					mFormPanel.setDisabled(false);
				}
				else
				{
					mFormPanel.setDisabled(true);
				}
			}});
        
        add(mFormPanel, eastLayoutData);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
		final AsyncCallback<Collection<JobDeclarationForm>> jobDeclarationCallback = new AsyncCallback<Collection<JobDeclarationForm>>() {
			public void onSuccess(Collection<JobDeclarationForm> jobDeclarations) {

				mStore.removeAll();
				mStore.commitChanges();
				for (JobDeclarationForm jobDeclaration : jobDeclarations) {
					mStore.add(mRecordDef.createRecord(new Object[] { 
							jobDeclaration,
							jobDeclaration.getId(), 
							jobDeclaration.getName(),
							jobDeclaration.getClassName()
					}));
				}
				mStore.commitChanges();
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		getAdminModule().getAdminModuleService().getJobDeclarations(jobDeclarationCallback);
	}
}
