/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.scheduling;

import java.util.Collection;
import java.util.HashSet;

import org.appspy.admin.client.AbstractPanel;
import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.scheduling.form.JobDeclarationForm;
import org.appspy.admin.client.scheduling.form.JobDeclarationParamForm;
import org.appspy.admin.client.scheduling.form.JobScheduleForm;
import org.appspy.admin.client.scheduling.form.JobScheduleParamForm;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.ComboBox;
import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.Label;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.event.ComboBoxListenerAdapter;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtextux.client.widgets.grid.plugins.GridSearchPlugin;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListJobSchedulePanel extends AbstractPanel {

	public static final String TAB_ID = "ListJobSchedulePanel";
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[] { 
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("jobDeclarationName"), 
			new StringFieldDef("jobDeclarationId"), 
			new StringFieldDef("cronExpression")});
	
	protected RecordDef mJobDeclarationRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name"), 
			new StringFieldDef("className")});

	protected EditorGridPanel mGridPanel = null;
	protected FormPanel mFormPanel = null;
	
	protected Store mStore = null;
	protected Store mJobDeclarationStore = null;
	
	public ListJobSchedulePanel(AdminModule adminModule) {
		
		super(adminModule);

		setTitle("Job Schedules");

		setLayout(new BorderLayout());
		setAutoScroll(true);

		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig("NAME", "jobDeclarationName", 100, true);
		ColumnConfig cronExpressionColumnConfig = new ColumnConfig("CRON EXPRESSION", "cronExpression", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				nameColumnConfig,
				cronExpressionColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);

		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(1);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		mStore = new Store(mRecordDef);
		mGridPanel.setStore(mStore);
		
		mJobDeclarationStore = new Store(mJobDeclarationRecordDef);

		Toolbar toolbar = new Toolbar();
		ToolbarButton newButton = new ToolbarButton("Add", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New job schedule");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(600);
				window.setHeight(400);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);
				
				final JobScheduleForm jobSchedule = new JobScheduleForm();
				final Record record = mRecordDef.createRecord(new Object[]{
						jobSchedule,
						null, 
						"", 
						null,
						""});
				
		        final ComboBox jobDeclarationComboBox = new ComboBox("Job Declaration", "jobDeclarationId");
		        formPanel.add(jobDeclarationComboBox, new AnchorLayoutData("100%"));
		        jobDeclarationComboBox.setStore(mJobDeclarationStore);
		        jobDeclarationComboBox.setValueField("id");
		        jobDeclarationComboBox.setDisplayField("name");
		        jobDeclarationComboBox.setMode(ComboBox.LOCAL);
		        jobDeclarationComboBox.setForceSelection(true);
		        jobDeclarationComboBox.setDisableKeyFilter(true);
		        jobDeclarationComboBox.setLazyRender(true);
		        jobDeclarationComboBox.setTriggerAction(ComboBox.ALL);

				TextField cronExpressionTextField = new TextField("Cron Expression", "cronExpression");
				formPanel.add(cronExpressionTextField, new AnchorLayoutData("100%"));
				
				Label cronHelpLabel = new Label();
				cronHelpLabel.setHtml("<a target='_blank' href='http://www.opensymphony.com/quartz/wikidocs/CronTriggers%20Tutorial.html'>Get help with cron expressions</a>");
				formPanel.add(cronHelpLabel, new AnchorLayoutData("100%"));

		        final ListJobScheduleParamPanel listParamPanel = new ListJobScheduleParamPanel(getAdminModule());
		        formPanel.add(listParamPanel, new AnchorLayoutData("100% -100"));
		        formPanel.setHeight(200);
		        
		        jobDeclarationComboBox.addListener(new ComboBoxListenerAdapter(){

                    @Override
                    public void onChange(Field field, Object newVal, Object oldVal)
                    {
						JobDeclarationForm jobDeclaration = (JobDeclarationForm) mJobDeclarationStore.query("id", jobDeclarationComboBox.getValueAsString())[0].getAsObject("object");
						HashSet<JobScheduleParamForm> params = new HashSet<JobScheduleParamForm>();
						for (JobDeclarationParamForm jobDeclarationParam : jobDeclaration.getParams())
						{
							JobScheduleParamForm jobScheduleParamForm = new JobScheduleParamForm();
							jobScheduleParamForm.setJobDeclarationParam(jobDeclarationParam);
							jobScheduleParamForm.setJobSchedule(jobSchedule);
							params.add(jobScheduleParamForm);
						}
						jobSchedule.setParams(params);
						listParamPanel.setParameters(jobSchedule.getParams());
                    }
		        	
		        });

		        final Button createButton = new Button("Create");
				formPanel.addButton(createButton);
				
				formPanel.getForm().loadRecord(record);

				createButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						jobSchedule.setCronExpression(record.getAsString("cronExpression"));
						JobDeclarationForm jobDeclaration = (JobDeclarationForm) mJobDeclarationStore.query("id", jobDeclarationComboBox.getValueAsString())[0].getAsObject("object");
						record.set("jobDeclarationName", jobDeclaration.getName());
						jobSchedule.setJobDeclaration(jobDeclaration);
						jobSchedule.setParams(listParamPanel.getParameters());
						for (JobScheduleParamForm param : listParamPanel.getParameters()) {
							param.setJobSchedule(jobSchedule);
						}
						final AsyncCallback<JobScheduleForm> callback = new AsyncCallback<JobScheduleForm>(){

							public void onFailure(Throwable caught) {
								MessageBox.alert("Unable to update job schedule", caught.getMessage());
							}

							public void onSuccess(JobScheduleForm result) {
								record.set("object", result);
								record.set("id", result.getId());
								mStore.add(record);
								record.commit();
								
								window.close();
							}
						};
						
						getAdminModule().getAdminModuleService().saveJobSchedule(jobSchedule, callback);
					}});
				
				window.add(formPanel);
				window.show();
			}
		});
		newButton.setIcon("images/silk/add.gif");
		toolbar.addButton(newButton);

		toolbar.addSeparator();

		ToolbarButton delButton = new ToolbarButton("Del", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				try {
					final Record record = mGridPanel.getSelectionModel().getSelected();
					if (record != null)
					{
						MessageBox.confirm("Confirm", "Are you sure you want to delete this job schedule ?",
								new MessageBox.ConfirmCallback() {

									public void execute(String btnID) {
										if (btnID.equals("yes")) {
											JobScheduleForm jobSchedule = (JobScheduleForm) record.getAsObject("object");
											getAdminModule().getAdminModuleService().deleteJobSchedule(jobSchedule, new AsyncCallback<JobScheduleForm>(){

												public void onFailure(Throwable caught) {
													MessageBox.alert("Unable to delete job schedule", caught.getLocalizedMessage());
												}

												public void onSuccess(JobScheduleForm result) {
													mStore.remove(record);
												}});
										}
									}
								});
					}
				} catch (Exception ex) {
					MessageBox.alert(ex.getMessage());
				}
			}
		});
		delButton.setIcon("images/silk/delete.gif");
		toolbar.addButton(delButton);
		
		toolbar.addSeparator();
		toolbar.addSpacer();
		
		ToolbarButton refreshButton = new ToolbarButton("Refresh", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				refreshDatas();
			}});
		refreshButton.setIcon("images/icons/fam/table_refresh.png");
		toolbar.addButton(refreshButton);

		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		GridSearchPlugin gridSearch = new GridSearchPlugin(GridSearchPlugin.TOP);  
		gridSearch.setMode(GridSearchPlugin.LOCAL);  
		mGridPanel.addPlugin(gridSearch);  

        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 5, 0, 5));
        centerLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        centerLayoutData.setMinSize(200);
        centerLayoutData.setMinWidth(200);
        
        add(mGridPanel, centerLayoutData);

        mFormPanel = new FormPanel();
        mFormPanel.setPaddings(10, 10, 10, 10);
        mFormPanel.setFrame(true);
        mFormPanel.setAutoScroll(true);
        mFormPanel.setCollapsible(true);
        mFormPanel.setWidth(400);
        mFormPanel.setLabelWidth(100);
        
        mFormPanel.setTitle("Job Schedule Properties");
        mFormPanel.setDisabled(true);
        
        final ComboBox jobDeclarationComboBox = new ComboBox("Job Declaration", "jobDeclarationId", 120);
        mFormPanel.add(jobDeclarationComboBox, new AnchorLayoutData("98%"));
        jobDeclarationComboBox.setStore(mJobDeclarationStore);
        jobDeclarationComboBox.setValueField("id");
        jobDeclarationComboBox.setDisplayField("name");
        jobDeclarationComboBox.setMode(ComboBox.LOCAL);
        jobDeclarationComboBox.setForceSelection(true);
        jobDeclarationComboBox.setDisableKeyFilter(true);
        jobDeclarationComboBox.setLazyRender(true);
        jobDeclarationComboBox.setTriggerAction(ComboBox.ALL);
        
        final TextField cronExpressionTextField = new TextField("Cron Expression", "cronExpression", 120);
        mFormPanel.add(cronExpressionTextField, new AnchorLayoutData("98%"));
        
		Label cronHelpLabel = new Label();
		cronHelpLabel.setHtml("<a target='_blank' href='http://www.opensymphony.com/quartz/wikidocs/CronTriggers%20Tutorial.html'>Get help with cron expressions</a>");
		mFormPanel.add(cronHelpLabel, new AnchorLayoutData("100%"));

        final ListJobScheduleParamPanel listParamPanel = new ListJobScheduleParamPanel(getAdminModule());
        mFormPanel.add(listParamPanel, new AnchorLayoutData("98% -100"));

        jobDeclarationComboBox.addListener(new ComboBoxListenerAdapter(){

            @Override
            public void onChange(Field field, Object newVal, Object oldVal)
            {
            	JobScheduleForm jobSchedule = (JobScheduleForm) mGridPanel.getSelectionModel().getSelected().getAsObject("object");
				JobDeclarationForm jobDeclaration = (JobDeclarationForm) mJobDeclarationStore.query("id", jobDeclarationComboBox.getValueAsString())[0].getAsObject("object");
				HashSet<JobScheduleParamForm> params = new HashSet<JobScheduleParamForm>();
				for (JobDeclarationParamForm jobDeclarationParam : jobDeclaration.getParams())
				{
					JobScheduleParamForm jobScheduleParamForm = new JobScheduleParamForm();
					jobScheduleParamForm.setJobDeclarationParam(jobDeclarationParam);
					jobScheduleParamForm.setJobSchedule(jobSchedule);
					params.add(jobScheduleParamForm);
				}
				jobSchedule.setParams(params);
				listParamPanel.setParameters(jobSchedule.getParams());
            }
        	
        });

        final Button commitButton = new Button("Save");
        mFormPanel.addButton(commitButton);
        
        final Button cancelButton = new Button("Cancel");
        mFormPanel.addButton(cancelButton);
        
        commitButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				final Record record = mGridPanel.getSelectionModel().getSelected();
				
				final AsyncCallback<JobScheduleForm> callback = new AsyncCallback<JobScheduleForm>(){

					public void onFailure(Throwable caught) {
						MessageBox.alert("Unable to update job schedule", caught.getMessage());
					}

					public void onSuccess(JobScheduleForm result) {
						record.set("object", result);
						record.commit();
					}
				};
				
				mFormPanel.getForm().updateRecord(record);
				final JobScheduleForm jobSchedule = (JobScheduleForm) record.getAsObject("object");
				JobDeclarationForm jobDeclaration = (JobDeclarationForm) mJobDeclarationStore.query("id", jobDeclarationComboBox.getValue())[0].getAsObject("object");
				record.set("jobDeclarationName", jobDeclaration.getName());
				jobSchedule.setJobDeclaration(jobDeclaration);
				jobSchedule.setCronExpression(record.getAsString("cronExpression"));
				jobSchedule.setParams(listParamPanel.getParameters());
				for (JobScheduleParamForm param : listParamPanel.getParameters()) {
					param.setJobSchedule(jobSchedule);
				}
				getAdminModule().getAdminModuleService().updateJobSchedule(jobSchedule, callback);
			}});
        
        cancelButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				mFormPanel.getForm().loadRecord(mGridPanel.getSelectionModel().getSelected());
				listParamPanel.setParameters(((JobScheduleForm)mGridPanel.getSelectionModel().getSelected().getAsObject("object")).getParams());
			}});
        
        
		BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
        eastLayoutData.setMargins(new Margins(5, 0, 5, 5));
        eastLayoutData.setMinWidth(400);
        eastLayoutData.setSplit(true);
        
        mGridPanel.getSelectionModel().addListener(new RowSelectionListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter#onSelectionChange(com.gwtext.client.widgets.grid.RowSelectionModel)
			 */
			@Override
			public void onSelectionChange(RowSelectionModel sm) {
				if (sm.getSelected() != null)
				{
					mFormPanel.getForm().loadRecord(sm.getSelected());
					listParamPanel.setParameters(((JobScheduleForm)sm.getSelected().getAsObject("object")).getParams());
					mFormPanel.setDisabled(false);
				}
				else
				{
					mFormPanel.setDisabled(true);
				}
			}});
        
        add(mFormPanel, eastLayoutData);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
		final AsyncCallback<Collection<JobScheduleForm>> jobScheduleCallback = new AsyncCallback<Collection<JobScheduleForm>>() {
			public void onSuccess(Collection<JobScheduleForm> jobSchedules) {

				mStore.removeAll();
				mStore.commitChanges();
				for (JobScheduleForm jobSchedule : jobSchedules) {
					mStore.add(mRecordDef.createRecord(new Object[] { 
							jobSchedule,
							jobSchedule.getId(), 
							jobSchedule.getJobDeclaration().getName(),
							jobSchedule.getJobDeclaration().getId(),
							jobSchedule.getCronExpression()
					}));
				}
				mStore.commitChanges();
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		final AsyncCallback<Collection<JobDeclarationForm>> jobDeclarationCallback = new AsyncCallback<Collection<JobDeclarationForm>>() {
			public void onSuccess(Collection<JobDeclarationForm> jobDeclarations) {

				mJobDeclarationStore.removeAll();
				mJobDeclarationStore.commitChanges();
				for (JobDeclarationForm jobDeclaration : jobDeclarations) {
					mJobDeclarationStore.add(mJobDeclarationRecordDef.createRecord(new Object[] { 
							jobDeclaration,
							jobDeclaration.getId(), 
							jobDeclaration.getName(),
							jobDeclaration.getClassName()
					}));
				}
				mJobDeclarationStore.commitChanges();
				
				getAdminModule().getAdminModuleService().getJobSchedules(jobScheduleCallback);
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		getAdminModule().getAdminModuleService().getJobDeclarations(jobDeclarationCallback);
	}
}
