/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.scheduling;

import java.util.HashSet;
import java.util.Set;

import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.paramchooser.ParamChooser;
import org.appspy.admin.client.reports.form.ReportParamChooserForm;
import org.appspy.admin.client.scheduling.form.JobScheduleParamForm;
import org.appspy.admin.client.util.ParamChooserFactory;

import com.google.gwt.core.client.GWT;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.Hidden;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.TriggerField;
import com.gwtext.client.widgets.form.event.FieldListener;
import com.gwtext.client.widgets.form.event.FieldListenerAdapter;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.GridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtext.client.widgets.layout.RowLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListJobScheduleParamPanel extends Panel {
	
	protected AdminModule mAdminModule = null;
	
	protected GridPanel mGridPanel = null;
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[]{
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"),
			new StringFieldDef("name"),
			new StringFieldDef("value")
	});
	
	protected Store mStore = null;
	
	public ListJobScheduleParamPanel(AdminModule adminModule)
	{
		mAdminModule = adminModule;
		
		setLayout(new RowLayout());
		
		mStore = new Store(mRecordDef);
		
		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig("NAME", "name", 100, true);
		ColumnConfig valueColumnConfig = new ColumnConfig("VALUE", "value", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				nameColumnConfig,
				valueColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);

		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);
		mGridPanel.setStore(mStore);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(1);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		Toolbar toolbar = new Toolbar();

		ToolbarButton editButton = new ToolbarButton("Edit", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("Edit Parameter");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(450);
				window.setHeight(350);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);

				final Record record = mGridPanel.getSelectionModel().getSelected();
				final JobScheduleParamForm param = (JobScheduleParamForm) record.getAsObject("object");
				
				final FieldListener fieldListener = new FieldListenerAdapter(){
					@Override
					public void onRender(Component component) {
						formPanel.getForm().loadRecord(record);
					}};
				TextField nameTextField = new TextField("Name", "name");
				nameTextField.setReadOnly(true);
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));
				nameTextField.addListener(fieldListener);
				
		        final Hidden defaultField = new Hidden("value", "value");
		        formPanel.add(defaultField);
		        defaultField.addListener(fieldListener);
		        
		        final TriggerField chooserField = new TriggerField(){

					@Override
					protected void onTriggerClick(EventObject event) {
						ReportParamChooserForm reportParamChooser = param.getJobDeclarationParam().getChooser();
						final Window chooserWindow = new Window();
						chooserWindow.setTitle("Choose value");
						chooserWindow.setClosable(true);
						chooserWindow.setFrame(true);
						chooserWindow.setWidth(450);
						chooserWindow.setHeight(350);
						chooserWindow.setAutoScroll(true);
						chooserWindow.setPlain(true);
						chooserWindow.setLayout(new FitLayout());
						chooserWindow.setCloseAction(Window.CLOSE);
						chooserWindow.setModal(true);
						
						final TriggerField f = this;

						try
						{
							final ParamChooser paramChooser = new ParamChooserFactory().getParamChooser(reportParamChooser.getChooserAdminClass());
							paramChooser.setAdminModule(mAdminModule);
							paramChooser.setValue(defaultField.getValueAsString());
							chooserWindow.add(paramChooser.getChooserComponent());
							chooserWindow.addButton(new Button("OK", new ButtonListenerAdapter(){

								@Override
								public void onClick(Button button, EventObject e) {
									defaultField.setValue(paramChooser.getValue());
									paramChooser.updateTextField(f);
									chooserWindow.destroy();
								}}));
							chooserWindow.addButton(new Button("Cancel", new ButtonListenerAdapter(){

								@Override
								public void onClick(Button button, EventObject e) {
									chooserWindow.destroy();
								}}));
							
							chooserWindow.show();
						}
						catch (Exception ex)
						{
							GWT.log(ex.getMessage(), ex);
						}
					}};
				chooserField.setLabel("Value");
				chooserField.setReadOnly(true);
		        formPanel.add(chooserField, new AnchorLayoutData("100%"));
		        
		        try
		        {
		        	ReportParamChooserForm reportParamChooser = ((JobScheduleParamForm) record.getAsObject("object")).getJobDeclarationParam().getChooser();
		        	final ParamChooser paramChooser = new ParamChooserFactory().getParamChooser(reportParamChooser.getChooserAdminClass());
		        	paramChooser.setAdminModule(mAdminModule);
		        	paramChooser.setValue(record.getAsString("value"));
		        	paramChooser.updateTextField(chooserField);
		        }
		        catch (Exception ex)
		        {
		        	GWT.log(ex.getMessage(), ex);
		        }
		        final Button saveButton = new Button("Save");
				formPanel.addButton(saveButton);
								
				saveButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						param.setValue(record.getAsString("value"));
						
						record.commit();
						window.close();
					}});
				
				final Button cancelButton = new Button("Cancel");
				formPanel.addButton(cancelButton);
				
				cancelButton.addListener(new ButtonListenerAdapter(){
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().loadRecord(record);
					}});

				window.add(formPanel);
				window.show();
			}
		});
		editButton.setIcon("images/silk/add.gif");
		toolbar.addButton(editButton);

		toolbar.addSeparator();
		toolbar.addSpacer();
		
		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		
		mGridPanel.setTitle("Parameters");
		add(mGridPanel);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
	}
	
	public Set<JobScheduleParamForm> getParameters()
	{
		HashSet<JobScheduleParamForm> params = new HashSet<JobScheduleParamForm>();
		
		for (Record record : mStore.getRecords()) {
			params.add((JobScheduleParamForm) record.getAsObject("object"));
		}
		
		return params;
	}
	
	public void setParameters(Set<JobScheduleParamForm> params)
	{
		mStore.removeAll();
		mStore.commitChanges();
		
		for (JobScheduleParamForm param : params) {
			mStore.add(mRecordDef.createRecord(new Object[]{
					param,
					param.getId(),
					param.getJobDeclarationParam().getName(),
					param.getValue()
			}));
		}
		
		mStore.commitChanges();
	}
}
