/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.client.settings;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;

import org.appspy.admin.client.AbstractPanel;
import org.appspy.admin.client.AdminModule;
import org.appspy.admin.client.settings.form.EnvironmentForm;
import org.appspy.admin.client.settings.form.HostForm;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.ObjectFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.Toolbar;
import com.gwtext.client.widgets.ToolbarButton;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.Label;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.grid.BaseColumnConfig;
import com.gwtext.client.widgets.grid.ColumnConfig;
import com.gwtext.client.widgets.grid.ColumnModel;
import com.gwtext.client.widgets.grid.EditorGridPanel;
import com.gwtext.client.widgets.grid.GridPanel;
import com.gwtext.client.widgets.grid.RowSelectionModel;
import com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtextux.client.widgets.grid.plugins.GridSearchPlugin;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ListHostPanel extends AbstractPanel {

	public static final String TAB_ID = "ListHostPanel";
	
	protected RecordDef mRecordDef = new RecordDef(new FieldDef[] { 
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"), 
			new StringFieldDef("name"), 
			new StringFieldDef("alias"),
			new StringFieldDef("envList")});
	
	protected RecordDef mEnvRecordDef = new RecordDef(new FieldDef[] {
			new ObjectFieldDef("object"),
			new IntegerFieldDef("id"),
			new StringFieldDef("name")
	});
	
	protected EditorGridPanel mGridPanel = null;
	protected FormPanel mFormPanel = null;
	
	protected Store mStore = null;
	protected Store mEnvStore = null;
	
	public ListHostPanel(AdminModule adminModule) {
		
		super(adminModule);
		
		setTitle("Hosts");

		setLayout(new BorderLayout());
		setAutoScroll(true);

		ColumnConfig idColumnConfig = new ColumnConfig("ID", "id", 100, true);
		ColumnConfig codeColumnConfig = new ColumnConfig("NAME", "name", 100, true);
		ColumnConfig nameColumnConfig = new ColumnConfig("ALIAS", "alias", 100, true);
		ColumnConfig envsColumnConfig = new ColumnConfig("ENVIRONMENTS", "envList", 100, true);

		ColumnConfig[] columns = new ColumnConfig[] { 
				idColumnConfig, 
				codeColumnConfig, 
				nameColumnConfig,
				envsColumnConfig};

		ColumnModel columnModel = new ColumnModel(columns);
		columnModel.setDefaultSortable(true);
		columnModel.setHidden(0, true);
		
		mGridPanel = new EditorGridPanel();
		mGridPanel.setColumnModel(columnModel);

		mGridPanel.setFrame(true);
		mGridPanel.setStripeRows(true);
		mGridPanel.setAutoExpandColumn(2);
		mGridPanel.setAutoScroll(true);
		
		final RowSelectionModel rowSelectionModel = new RowSelectionModel();
		mGridPanel.setSelectionModel(rowSelectionModel);

		mStore = new Store(mRecordDef);
		mGridPanel.setStore(mStore);
		
		mEnvStore = new Store(mEnvRecordDef);

		Toolbar toolbar = new Toolbar();
		ToolbarButton newButton = new ToolbarButton("Add", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				final Window window = new Window();
				window.setTitle("New organization");
				window.setClosable(true);
				window.setFrame(true);
				window.setWidth(450);
				window.setHeight(350);
				window.setAutoScroll(true);
				window.setPlain(true);
				window.setLayout(new FitLayout());
				window.setCloseAction(Window.CLOSE);
				window.setModal(true);
				
				final FormPanel formPanel = new FormPanel();
				formPanel.setPaddings(10, 10, 10, 10);
				formPanel.setFrame(true);

				final HostForm host = new HostForm();
				final Record record = mRecordDef.createRecord(new Object[]{
						host,
						null,
						"",
						"",
						""
				});
				
				TextField nameTextField = new TextField("Name", "name");
				formPanel.add(nameTextField, new AnchorLayoutData("100%"));
				
				TextField aliasTextField = new TextField("Alias", "alias");
				formPanel.add(aliasTextField, new AnchorLayoutData("100%"));

		        Label envListLabel = new Label("Environments :");
		        formPanel.add(envListLabel);
		        
				final GridPanel envListGridPanel = new GridPanel();
				BaseColumnConfig[] envListColumnConfig = new BaseColumnConfig[]{
					new ColumnConfig("ID", "id", 20, true),
					new ColumnConfig("NAME", "name", 100, true)
				};
		        
		        ColumnModel envListColumnModel = new ColumnModel(envListColumnConfig);
		        envListColumnModel.setHidden(0, true);
		        envListGridPanel.setAutoExpandColumn(1);
		        envListGridPanel.setColumnModel(envListColumnModel);
		        envListGridPanel.setHideColumnHeader(true);
		        
		        envListGridPanel.setStore(mEnvStore);
		        
		        formPanel.add(envListGridPanel, new AnchorLayoutData("100% -70"));

		        final Button createButton = new Button("Create");
				formPanel.addButton(createButton);
				
				formPanel.getForm().loadRecord(record);
				
				createButton.addListener(new ButtonListenerAdapter(){

					/* (non-Javadoc)
					 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
					 */
					@Override
					public void onClick(Button button, EventObject e) {
						formPanel.getForm().updateRecord(record);
						host.setName(record.getAsString("name"));
						host.setAlias(record.getAsString("alias"));
						host.setEnvironments(new ArrayList<EnvironmentForm>());
						Record[] sel = envListGridPanel.getSelectionModel().getSelections();
						for (int i = 0; i < sel.length; i++) {
							host.getEnvironments().add((EnvironmentForm) sel[i].getAsObject("object"));
						}
						record.set("envList", formatEnvs(host.getEnvironments()));
						final AsyncCallback<HostForm> callback = new AsyncCallback<HostForm>(){

							public void onFailure(Throwable caught) {
								MessageBox.alert("Technical problem", "Unable to create organization");
							}

							public void onSuccess(HostForm result) {
								record.set("object", result);
								record.set("id", result.getId());
								mStore.add(record);
								record.commit();
								window.close();
							}
						};
						
						getAdminModule().getAdminModuleService().saveHost(host, callback);
					}});
				
				window.add(formPanel);
				window.show();

			}
		});
		newButton.setIcon("images/silk/add.gif");
		toolbar.addButton(newButton);

		toolbar.addSeparator();

		ToolbarButton delButton = new ToolbarButton("Del", new ButtonListenerAdapter() {
			public void onClick(Button button, EventObject e) {
				try {
					final Record record = mGridPanel.getSelectionModel().getSelected();
					if (record != null)
					{
						MessageBox.confirm("Confirm", "Be carefull before deleting a host. Do you confirm ?",
								new MessageBox.ConfirmCallback() {

									public void execute(String btnID) {
										if (btnID.equals("yes")) {
											HostForm host = (HostForm) record.getAsObject("object");
											getAdminModule().getAdminModuleService().deleteHost(host, new AsyncCallback<HostForm>(){

												public void onFailure(Throwable caught) {
													MessageBox.alert("Unable to delete host", caught.getLocalizedMessage());
												}

												public void onSuccess(HostForm result) {
													mStore.remove(record);
												}});
										}
									}
								});
					}
				} catch (Exception ex) {
					MessageBox.alert(ex.getMessage());
				}
			}
		});
		delButton.setIcon("images/silk/delete.gif");
		toolbar.addButton(delButton);
		
		toolbar.addSeparator();
		toolbar.addSpacer();
		
		ToolbarButton refreshButton = new ToolbarButton("Refresh", new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				refreshDatas();
			}});
		refreshButton.setIcon("images/icons/fam/table_refresh.png");
		toolbar.addButton(refreshButton);

		mGridPanel.setTopToolbar(toolbar);
		
		toolbar.addFill();
		GridSearchPlugin gridSearch = new GridSearchPlugin(GridSearchPlugin.TOP);  
		gridSearch.setMode(GridSearchPlugin.LOCAL);  
		mGridPanel.addPlugin(gridSearch);
		
        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 5, 0, 5));
        centerLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        centerLayoutData.setMinSize(50);
        centerLayoutData.setMinWidth(300);
        
        mGridPanel.setWidth(400);
        
        add(mGridPanel, centerLayoutData);

        mFormPanel = new FormPanel();
        mFormPanel.setPaddings(10, 10, 10, 10);
        mFormPanel.setFrame(true);
        mFormPanel.setAutoScroll(true);
        mFormPanel.setCollapsible(true);
        mFormPanel.setWidth(250);
        mFormPanel.setLabelWidth(50);
        
        mFormPanel.setTitle("Host Properties");
        mFormPanel.setDisabled(true);
        
        final TextField nameTextField = new TextField("Name", "name", 120);
        mFormPanel.add(nameTextField, new AnchorLayoutData("100%"));
        
        final TextField aliasTextField = new TextField("Alias", "alias", 120);
        mFormPanel.add(aliasTextField, new AnchorLayoutData("100%"));
        
        final Label envListLabel = new Label("Environments :");
        mFormPanel.add(envListLabel);
        
		final GridPanel envListGridPanel = new GridPanel();
		BaseColumnConfig[] envListColumnConfig = new BaseColumnConfig[]{
			new ColumnConfig("ID", "id", 20, true),
			new ColumnConfig("NAME", "name", 100, true)
		};
        
        ColumnModel envListColumnModel = new ColumnModel(envListColumnConfig);
        envListColumnModel.setHidden(0, true);
        envListGridPanel.setAutoExpandColumn(1);
        envListGridPanel.setColumnModel(envListColumnModel);
        envListGridPanel.setHideColumnHeader(true);
        
        envListGridPanel.setStore(mEnvStore);
        
        mFormPanel.add(envListGridPanel, new AnchorLayoutData("100% -70"));
        
        final Button commitButton = new Button("Save");
        mFormPanel.addButton(commitButton);
        
        final Button cancelButton = new Button("Cancel");
        mFormPanel.addButton(cancelButton);

        commitButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				final Record record = mGridPanel.getSelectionModel().getSelected();
				
				final AsyncCallback<HostForm> callback = new AsyncCallback<HostForm>(){

					public void onFailure(Throwable caught) {
						MessageBox.alert("Technical problem", "Unable to update host");
					}

					public void onSuccess(HostForm result) {
						record.commit();
					}
				};
				
				mFormPanel.getForm().updateRecord(record);
				final HostForm host = (HostForm) record.getAsObject("object");
				host.setName(record.getAsString("name"));
				host.setAlias(record.getAsString("alias"));
				host.getEnvironments().clear();
				Record[] sel = envListGridPanel.getSelectionModel().getSelections();
				for (int i = 0; i < sel.length; i++) {
					host.getEnvironments().add((EnvironmentForm) sel[i].getAsObject("object"));
				}
				record.set("envList", formatEnvs(host.getEnvironments()));
				if (Arrays.asList(record.getModifiedFields()).contains("name"))
				{
					MessageBox.ConfirmCallback confirmCallback = new MessageBox.ConfirmCallback(){
						public void execute(String btnID) {
							if (btnID.equals("yes")) {
								getAdminModule().getAdminModuleService().updateHost(host, callback);
							}
							else
							{
								record.reject();
								mFormPanel.getForm().loadRecord(record);
							}
						}
					};
					MessageBox.confirm("Confirm", "Be carefull before changing the name of a host. Do you confirm ?",
							confirmCallback);
				}
				else
				{
					getAdminModule().getAdminModuleService().updateHost(host, callback);
				}
			}});
        
        cancelButton.addListener(new ButtonListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
			 */
			@Override
			public void onClick(Button button, EventObject e) {
				mFormPanel.getForm().loadRecord(mGridPanel.getSelectionModel().getSelected());
				HostForm host = (HostForm) mGridPanel.getSelectionModel().getSelected().getAsObject("object");
				ArrayList<Record> selectedRecords = new ArrayList<Record>();
				for (EnvironmentForm env : host.getEnvironments()) {
					selectedRecords.add(mEnvStore.query("id", env.getId().toString())[0]);
				}
				envListGridPanel.getSelectionModel().selectRecords(selectedRecords.toArray(new Record[0]));
			}});
        
        
		BorderLayoutData eastLayoutData = new BorderLayoutData(RegionPosition.EAST);
        eastLayoutData.setMargins(new Margins(5, 0, 5, 5));
        eastLayoutData.setMinWidth(220);
        eastLayoutData.setSplit(true);
        
        mGridPanel.getSelectionModel().addListener(new RowSelectionListenerAdapter(){

			/* (non-Javadoc)
			 * @see com.gwtext.client.widgets.grid.event.RowSelectionListenerAdapter#onSelectionChange(com.gwtext.client.widgets.grid.RowSelectionModel)
			 */
			@Override
			public void onSelectionChange(RowSelectionModel sm) {
				if (sm.getSelected() != null)
				{
					mFormPanel.getForm().loadRecord(sm.getSelected());
					HostForm host = (HostForm) sm.getSelected().getAsObject("object");
					ArrayList<Record> selectedRecords = new ArrayList<Record>();
					for (EnvironmentForm env : host.getEnvironments()) {
						selectedRecords.add(mEnvStore.query("id", env.getId().toString())[0]);
					}
					envListGridPanel.getSelectionModel().selectRecords(selectedRecords.toArray(new Record[0]));
					mFormPanel.setDisabled(false);
				}
				else
				{
					mFormPanel.setDisabled(true);
				}
			}});
        
        add(mFormPanel, eastLayoutData);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.gwtext.client.widgets.Component#initComponent()
	 */
	@Override
	protected void initComponent() {
		refreshDatas();
	}

	public void refreshDatas()
	{
		final AsyncCallback<Collection<HostForm>> envCallback = new AsyncCallback<Collection<HostForm>>() {
			public void onSuccess(Collection<HostForm> hosts) {

				mStore.removeAll();
				mStore.commitChanges();
				for (HostForm host : hosts) {
					mStore.add(mRecordDef.createRecord(new Object[] { 
							host,
							host.getId(), 
							host.getName(), 
							host.getAlias(), 
							formatEnvs(host.getEnvironments())}));
				}
				mStore.commitChanges();
			}

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
		};

		AsyncCallback<Collection<EnvironmentForm>> orgCallback = new AsyncCallback<Collection<EnvironmentForm>>(){

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}

			public void onSuccess(Collection<EnvironmentForm> result) {
				
				mEnvStore.removeAll();
				for (EnvironmentForm env : result) {
					mEnvStore.add(mEnvRecordDef.createRecord(new Object[]{
							env,
							env.getId(), 
							formatEnv(env)}));
				}
				mEnvStore.commitChanges();
				getAdminModule().getAdminModuleService().getHosts(envCallback);
			}};
		
		getAdminModule().getAdminModuleService().getEnvironments(orgCallback);
	}
	
	public String formatEnvs(Collection<EnvironmentForm> envs)
	{
		String result = "";
		for (EnvironmentForm env : envs) {
			result = result + formatEnv(env) + ",";
			}
		result = result.substring(0, result.length() - 1);
		
		return result;
	}
	
	public String formatEnv(EnvironmentForm env)
	{
		String result = "";
		if (env.getOrganization() != null)
		{
			result = env.getOrganization().getName() + "/" + env.getName();
		}
		else
		{
			result = env.getName();
		}
		return result;
	}
}
