/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.server.rpc;

import java.util.ArrayList;
import java.util.Collection;

import net.sf.dozer.util.mapping.DozerBeanMapper;

import org.appspy.admin.client.reports.form.DashboardForm;
import org.appspy.admin.client.reports.form.ReportCategoryForm;
import org.appspy.admin.client.reports.form.ReportForm;
import org.appspy.admin.client.reports.form.ReportParamChooserForm;
import org.appspy.admin.client.reports.form.ReportingSystemForm;
import org.appspy.admin.client.rpc.AdminModuleService;
import org.appspy.admin.client.scheduling.form.JobDeclarationForm;
import org.appspy.admin.client.scheduling.form.JobScheduleForm;
import org.appspy.admin.client.settings.form.ApplicationForm;
import org.appspy.admin.client.settings.form.ApplicationVersionForm;
import org.appspy.admin.client.settings.form.EnvironmentForm;
import org.appspy.admin.client.settings.form.HostForm;
import org.appspy.admin.client.settings.form.OrganizationForm;
import org.appspy.admin.service.ReportService;
import org.appspy.admin.service.SchedulingService;
import org.appspy.admin.service.SettingsService;
import org.appspy.server.bo.Application;
import org.appspy.server.bo.ApplicationVersion;
import org.appspy.server.bo.Dashboard;
import org.appspy.server.bo.Environment;
import org.appspy.server.bo.Host;
import org.appspy.server.bo.Organization;
import org.appspy.server.bo.Report;
import org.appspy.server.bo.ReportCategory;
import org.appspy.server.bo.ReportParamChooser;
import org.appspy.server.bo.ReportingSystem;
import org.appspy.server.bo.scheduler.JobDeclaration;
import org.appspy.server.bo.scheduler.JobSchedule;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class AdminModuleServiceImpl extends RemoteServiceServlet implements
		AdminModuleService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.settings.rpc.EnvironmentService#saveEnvironment(org.appspy.admin.client.settings.form.EnvironmentForm)
	 */
	public EnvironmentForm saveEnvironment(EnvironmentForm envForm) {
		
		Environment result = getSettingsService().createEnvironment((Environment) getBeanMapper().map(envForm, Environment.class));

		return (EnvironmentForm) getBeanMapper().map(result, EnvironmentForm.class);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#deleteEnvironment(org.appspy.admin.client.settings.form.EnvironmentForm)
	 */
	public void deleteEnvironment(EnvironmentForm envForm) {
		getSettingsService().deleteEnvironment((Environment) getBeanMapper().map(envForm, Environment.class));
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getEnvironments()
	 */
	public Collection<EnvironmentForm> getEnvironments() {
		Collection<Environment> envs = getSettingsService().getEnvironments();
		
		Collection<EnvironmentForm> result = new ArrayList<EnvironmentForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (Environment env : envs) {
			result.add((EnvironmentForm) beanMapper.map(env, EnvironmentForm.class));
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateEnvironment(org.appspy.admin.client.settings.form.EnvironmentForm)
	 */
	public EnvironmentForm updateEnvironment(EnvironmentForm envForm) {
		Environment result = getSettingsService().updateEnvironment((Environment) getBeanMapper().map(envForm, Environment.class));

		return (EnvironmentForm) getBeanMapper().map(result, EnvironmentForm.class);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#saveOrganization(org.appspy.admin.client.settings.form.OrganizationForm)
	 */
	public OrganizationForm saveOrganization(OrganizationForm orgForm) {
		Organization result = getSettingsService().createOrganization((Organization) getBeanMapper().map(orgForm, Organization.class));

		return (OrganizationForm) getBeanMapper().map(result, OrganizationForm.class);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateOrganization(org.appspy.admin.client.settings.form.OrganizationForm)
	 */
	public OrganizationForm updateOrganization(OrganizationForm orgForm) {
		Organization result = getSettingsService().updateOrganization((Organization) getBeanMapper().map(orgForm, Organization.class));

		return (OrganizationForm) getBeanMapper().map(result, OrganizationForm.class);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateOrganization(org.appspy.admin.client.settings.form.OrganizationForm)
	 */
	public void deleteOrganization(OrganizationForm orgForm) {
		getSettingsService().deleteOrganization((Organization) getBeanMapper().map(orgForm, Organization.class));
	}
	

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getOrganizations()
	 */
	public Collection<OrganizationForm> getOrganizations() {
		Collection<Organization> organizations = getSettingsService().getOrganizations();
		
		Collection<OrganizationForm> result = new ArrayList<OrganizationForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (Organization organization : organizations) {
			result.add((OrganizationForm) beanMapper.map(organization, OrganizationForm.class));
		}
		
		return result;
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#deleteHost(org.appspy.admin.client.settings.form.HostForm)
	 */
	public void deleteHost(HostForm hostForm) {
		getSettingsService().deleteHost((Host) getBeanMapper().map(hostForm, Host.class));
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getHosts()
	 */
	public Collection<HostForm> getHosts() {
		Collection<Host> hosts = getSettingsService().getHosts();
		
		Collection<HostForm> result = new ArrayList<HostForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (Host host : hosts) {
			result.add((HostForm) beanMapper.map(host, HostForm.class));
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#saveHost(org.appspy.admin.client.settings.form.HostForm)
	 */
	public HostForm saveHost(HostForm hostForm) {
		Host result = getSettingsService().createHost((Host) getBeanMapper().map(hostForm, Host.class));

		return (HostForm) getBeanMapper().map(result, HostForm.class);

	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateHost(org.appspy.admin.client.settings.form.HostForm)
	 */
	public HostForm updateHost(HostForm hostForm) {
		Host result = getSettingsService().updateHost((Host) getBeanMapper().map(hostForm, Host.class));

		return (HostForm) getBeanMapper().map(result, HostForm.class);

	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#deleteApplication(org.appspy.admin.client.settings.form.ApplicationForm)
	 */
	public void deleteApplication(ApplicationForm appForm) {
		getSettingsService().deleteApplication((Application) getBeanMapper().map(appForm, Application.class));
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getApplications()
	 */
	public Collection<ApplicationForm> getApplications() {
		Collection<Application> applications = getSettingsService().getApplications();
		
		Collection<ApplicationForm> result = new ArrayList<ApplicationForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (Application application : applications) {
			result.add((ApplicationForm) beanMapper.map(application, ApplicationForm.class));
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#saveApplication(org.appspy.admin.client.settings.form.ApplicationForm)
	 */
	public ApplicationForm saveApplication(ApplicationForm appForm) {
		Application result = getSettingsService().createApplication((Application) getBeanMapper().map(appForm, Application.class));

		return (ApplicationForm) getBeanMapper().map(result, ApplicationForm.class);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateApplication(org.appspy.admin.client.settings.form.ApplicationForm)
	 */
	public ApplicationForm updateApplication(ApplicationForm appForm) {
		Application result = getSettingsService().updateApplication((Application) getBeanMapper().map(appForm, Application.class));

		return (ApplicationForm) getBeanMapper().map(result, ApplicationForm.class);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#deleteVersion(org.appspy.admin.client.settings.form.ApplicationVersionForm)
	 */
	public void deleteVersion(ApplicationVersionForm verForm) {
		getSettingsService().deleteVersion((ApplicationVersion) getBeanMapper().map(verForm, ApplicationVersion.class));
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getVersions()
	 */
	public Collection<ApplicationVersionForm> getVersions() {
		Collection<ApplicationVersion> versions = getSettingsService().getVersions();
		
		Collection<ApplicationVersionForm> result = new ArrayList<ApplicationVersionForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (ApplicationVersion version : versions) {
			result.add((ApplicationVersionForm) beanMapper.map(version, ApplicationVersionForm.class));
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#saveVersion(org.appspy.admin.client.settings.form.ApplicationVersionForm)
	 */
	public ApplicationVersionForm saveVersion(ApplicationVersionForm verForm) {
		ApplicationVersion result = getSettingsService().createVersion((ApplicationVersion) getBeanMapper().map(verForm, ApplicationVersion.class));

		return (ApplicationVersionForm) getBeanMapper().map(result, ApplicationVersionForm.class);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateVersion(org.appspy.admin.client.settings.form.ApplicationVersionForm)
	 */
	public ApplicationVersionForm updateVersion(ApplicationVersionForm verForm) {
		ApplicationVersion result = getSettingsService().updateVersion((ApplicationVersion) getBeanMapper().map(verForm, ApplicationVersion.class));

		return (ApplicationVersionForm) getBeanMapper().map(result, ApplicationVersionForm.class);
	}
	
	

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#deleteReportCategory(org.appspy.admin.client.report.form.ReportCategoryForm)
	 */
	public void deleteReportCategory(ReportCategoryForm catForm) {
		getReportService().deleteReportCategory((ReportCategory) getBeanMapper().map(catForm, ReportCategory.class));
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getReportCategories()
	 */
	public Collection<ReportCategoryForm> getReportCategories() {
		Collection<ReportCategory> cats = getReportService().getReportCategories();
		
		Collection<ReportCategoryForm> result = new ArrayList<ReportCategoryForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (ReportCategory cat : cats) {
			result.add((ReportCategoryForm) beanMapper.map(cat, ReportCategoryForm.class));
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#saveReportCategory(org.appspy.admin.client.report.form.ReportCategoryForm)
	 */
	public ReportCategoryForm saveReportCategory(ReportCategoryForm catForm) {
		ReportCategory result = getReportService().createReportCategory((ReportCategory) getBeanMapper().map(catForm, ReportCategory.class));

		return (ReportCategoryForm) getBeanMapper().map(result, ReportCategoryForm.class);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateReportCategory(org.appspy.admin.client.report.form.ReportCategoryForm)
	 */
	public ReportCategoryForm updateReportCategory(ReportCategoryForm catForm) {
		ReportCategory result = getReportService().updateReportCategory((ReportCategory) getBeanMapper().map(catForm, ReportCategory.class));

		return (ReportCategoryForm) getBeanMapper().map(result, ReportCategoryForm.class);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#deleteReportDescription(org.appspy.admin.client.report.form.ReportDescriptionForm)
	 */
	public void deleteReport(ReportForm reportForm) {
		getReportService().deleteReport((Report) getBeanMapper().map(reportForm, Report.class));
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getReportDescriptions()
	 */
	public Collection<ReportForm> getReports() {
		Collection<Report> reports = getReportService().getReports();
		
		Collection<ReportForm> result = new ArrayList<ReportForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (Report report : reports) {
			result.add((ReportForm) beanMapper.map(report, ReportForm.class));
		}
		
		return result;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#saveReportDescription(org.appspy.admin.client.report.form.ReportDescriptionForm)
	 */
	public ReportForm saveReport(ReportForm reportForm) {
		Report result = getReportService().createReport((Report) getBeanMapper().map(reportForm, Report.class));

		return (ReportForm) getBeanMapper().map(result, ReportForm.class);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateReportDescription(org.appspy.admin.client.report.form.ReportDescriptionForm)
	 */
	public ReportForm updateReport(ReportForm reportForm) {
		Report result = getReportService().updateReport((Report) getBeanMapper().map(reportForm, Report.class));

		return (ReportForm) getBeanMapper().map(result, ReportForm.class);
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getReportingSystems()
	 */
	public Collection<ReportingSystemForm> getReportingSystems() {
		Collection<ReportingSystem> reportingSystems = getReportService().getReportingSystems();
		
		Collection<ReportingSystemForm> result = new ArrayList<ReportingSystemForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (ReportingSystem sys : reportingSystems) {
			result.add((ReportingSystemForm) beanMapper.map(sys, ReportingSystemForm.class));
		}
		
		return result;
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#deleteDashboard(org.appspy.admin.client.reports.form.DashboardForm)
	 */
	public void deleteDashboard(DashboardForm dashboardForm) {
		getReportService().deleteDashboard((Dashboard) getBeanMapper().map(dashboardForm, Dashboard.class));
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getDashboards()
	 */
	public Collection<DashboardForm> getDashboards() {
		Collection<Dashboard> dashboards = getReportService().getDashboards();
		
		Collection<DashboardForm> result = new ArrayList<DashboardForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (Dashboard dashboard : dashboards) {
			result.add((DashboardForm) beanMapper.map(dashboard, DashboardForm.class));
		}
		
		return result;
	}
	
	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#getDashboard(java.lang.Long)
     */
    public DashboardForm getDashboard(Long dashboardId)
    {
	    return (DashboardForm) getBeanMapper().map(getReportService().findDashboardById(dashboardId), DashboardForm.class);
    }

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#saveDashboard(org.appspy.admin.client.reports.form.DashboardForm)
	 */
	public DashboardForm saveDashboard(DashboardForm dashboardForm) {
		Dashboard result = getReportService().createDashboard((Dashboard) getBeanMapper().map(dashboardForm, Dashboard.class));

		return (DashboardForm) getBeanMapper().map(result, DashboardForm.class);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#updateDashboard(org.appspy.admin.client.reports.form.DashboardForm)
	 */
	public DashboardForm updateDashboard(DashboardForm dashboardForm) {
		Dashboard result = getReportService().updateDashboard((Dashboard) getBeanMapper().map(dashboardForm, Dashboard.class));

		return (DashboardForm) getBeanMapper().map(result, DashboardForm.class);
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.rpc.AdminModuleService#getReportParamChoosers()
	 */
	public Collection<ReportParamChooserForm> getReportParamChoosers() {
		Collection<ReportParamChooser> reportParamChoosers = getReportService().getReportParamChoosers();
		
		Collection<ReportParamChooserForm> result = new ArrayList<ReportParamChooserForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (ReportParamChooser chooser : reportParamChoosers) {
			result.add((ReportParamChooserForm) beanMapper.map(chooser, ReportParamChooserForm.class));
		}
		
		return result;
	}
	
	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#deleteJobDeclaration(org.appspy.admin.client.scheduling.form.JobDeclarationForm)
     */
    public void deleteJobDeclaration(JobDeclarationForm jobDeclarationForm)
    {
		getSchedulingService().deleteJobDeclaration((JobDeclaration) getBeanMapper().map(jobDeclarationForm, JobDeclaration.class));
    }

	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#getJobDeclarations()
     */
    public Collection<JobDeclarationForm> getJobDeclarations()
    {
		Collection<JobDeclaration> jobDeclarations = getSchedulingService().getJobDeclarations();
		
		Collection<JobDeclarationForm> result = new ArrayList<JobDeclarationForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (JobDeclaration jobDeclaration : jobDeclarations) {
			result.add((JobDeclarationForm) beanMapper.map(jobDeclaration, JobDeclarationForm.class));
		}
		
		return result;
    }

	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#saveJobDeclaration(org.appspy.admin.client.scheduling.form.JobDeclarationForm)
     */
    public JobDeclarationForm saveJobDeclaration(JobDeclarationForm jobDeclarationForm) throws IllegalArgumentException
    {
    	try
    	{
    		JobDeclaration result = getSchedulingService().createJobDeclaration((JobDeclaration) getBeanMapper().map(jobDeclarationForm, JobDeclaration.class));
    		
    		return (JobDeclarationForm) getBeanMapper().map(result, JobDeclarationForm.class);
    	}
    	catch (Exception ex)
    	{
    		throw new IllegalArgumentException(ex.getMessage());
    	}
    }

	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#updateJobDeclaration(org.appspy.admin.client.scheduling.form.JobDeclarationForm)
     */
    public JobDeclarationForm updateJobDeclaration(JobDeclarationForm jobDeclarationForm)  throws IllegalArgumentException
    {
    	try
    	{
    		JobDeclaration result = getSchedulingService().updateJobDeclaration((JobDeclaration) getBeanMapper().map(jobDeclarationForm, JobDeclaration.class));
    		
    		return (JobDeclarationForm) getBeanMapper().map(result, JobDeclarationForm.class);
    	}
    	catch (Exception ex)
    	{
    		throw new IllegalArgumentException(ex.getMessage());
    	}
    }

	protected SettingsService getSettingsService() {
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
		SettingsService settingsService = (SettingsService) ctx.getBean("settingsService");
		return settingsService;
	}
	
	protected ReportService getReportService() {
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
		ReportService reportService = (ReportService) ctx.getBean("reportService");
		return reportService;
	}
	
	protected SchedulingService getSchedulingService() {
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
		SchedulingService schedulingService = (SchedulingService) ctx.getBean("schedulingService");
		return schedulingService;
	}
	
	protected DozerBeanMapper getBeanMapper()
	{
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
		DozerBeanMapper beanMapper = (DozerBeanMapper) ctx.getBean("beanMapper");
		return beanMapper;
	}

	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#deleteJobSchedule(org.appspy.admin.client.scheduling.form.JobScheduleForm)
     */
    public void deleteJobSchedule(JobScheduleForm jobScheduleForm)
    {
		getSchedulingService().deleteJobSchedule((JobSchedule) getBeanMapper().map(jobScheduleForm, JobSchedule.class));
	    
    }

	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#getJobSchedules()
     */
    public Collection<JobScheduleForm> getJobSchedules()
    {
		Collection<JobSchedule> jobSchedules = getSchedulingService().getJobSchedules();
		
		Collection<JobScheduleForm> result = new ArrayList<JobScheduleForm>();

		DozerBeanMapper beanMapper = getBeanMapper();
		for (JobSchedule jobSchedule : jobSchedules) {
			result.add((JobScheduleForm) beanMapper.map(jobSchedule, JobScheduleForm.class));
		}
		
		return result;
    }

	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#saveJobSchedule(org.appspy.admin.client.scheduling.form.JobScheduleForm)
     */
    public JobScheduleForm saveJobSchedule(JobScheduleForm jobScheduleForm) throws IllegalArgumentException
    {
    	try
    	{
    		JobSchedule result = getSchedulingService().createJobSchedule((JobSchedule) getBeanMapper().map(jobScheduleForm, JobSchedule.class));

    		return (JobScheduleForm) getBeanMapper().map(result, JobScheduleForm.class);
    	}
    	catch (Exception ex)
    	{
    		throw new IllegalArgumentException(ex.getMessage());
    	}
    }

	/* (non-Javadoc)
     * @see org.appspy.admin.client.rpc.AdminModuleService#updateJobSchedule(org.appspy.admin.client.scheduling.form.JobScheduleForm)
     */
    public JobScheduleForm updateJobSchedule(JobScheduleForm jobScheduleForm) throws IllegalArgumentException
    {
    	try
    	{
    		JobSchedule result = getSchedulingService().updateJobSchedule((JobSchedule) getBeanMapper().map(jobScheduleForm, JobSchedule.class));
    		
    		return (JobScheduleForm) getBeanMapper().map(result, JobScheduleForm.class);
    	}
    	catch (Exception ex)
    	{
    		throw new IllegalArgumentException(ex.getMessage());
    	}
    }

	
}
