/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.server.servlet;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.IOUtils;
import org.appspy.admin.service.ReportService;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import com.thoughtworks.xstream.XStream;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ImportReportServlet extends HttpServlet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@SuppressWarnings("unchecked")
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException
	{
		response.setContentType("text/plain");
		
		FileItem importUploadItem = null;

		FileItemFactory factory = new DiskFileItemFactory();
		ServletFileUpload upload = new ServletFileUpload(factory);
		
		try
		{
			for (Iterator<FileItem> iterator = upload.parseRequest(request).iterator(); iterator.hasNext();) {
				FileItem item = (FileItem) iterator.next();
				
				if (!item.isFormField() && "importFile".equals(item.getFieldName()))
				{
					importUploadItem = item;
				}
			}
			
			if (importUploadItem == null)
			{
				response.getOutputStream().println("{\"success\":false,\"errors\":[{\"Problem during file upload\"}]}");
				return;
			}
			
			byte[] importContent = importUploadItem.get();
			
			ZipInputStream zis = new ZipInputStream(new ByteArrayInputStream(importContent));
			ZipEntry ze = zis.getNextEntry();
			
			TreeMap<String, byte[]> reportContents = new TreeMap<String, byte[]>();
			List<Object> xmlDatas = new ArrayList<Object>();
			
			while (ze != null)
			{
				if (ze.getName().equals("report-datas.xml"))
				{
					XStream xstream = new XStream();
					xmlDatas = (List<Object>) xstream.fromXML(zis); 
				}
				else
				{
					byte[] reportContent = IOUtils.toByteArray(zis);
					reportContents.put(ze.getName(), reportContent);
				}
				
				ze = zis.getNextEntry();
			}
			
			zis.close();
			
			getReportService().importReports(xmlDatas, reportContents);
						
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		response.setContentType("text/xml");
		PrintWriter pw = response.getWriter();
		pw.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		pw.println("<response success=\"true\">");
		pw.println("<data>");
		pw.println("<responseMsg>this is a message</responseMsg>");
		pw.println("</data>");
		pw.println("</response>");

		pw.flush();
	}
	
	protected ReportService getReportService() {
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
		ReportService reportService = (ReportService) ctx.getBean("reportService");
		return reportService;
	}
}
