/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.admin.server.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.appspy.admin.service.ReportService;
import org.appspy.server.bo.Report;
import org.appspy.server.bo.ReportEntityListener;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class UploadReportServlet extends HttpServlet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@SuppressWarnings("unchecked")
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException
	{
		response.setContentType("text/plain");
		
		FileItem reportUploadItem = null;
		Long reportId = null;

		FileItemFactory factory = new DiskFileItemFactory();
		ServletFileUpload upload = new ServletFileUpload(factory);
		
		try
		{
			for (Iterator<FileItem> iterator = upload.parseRequest(request).iterator(); iterator.hasNext();) {
				FileItem item = (FileItem) iterator.next();
				
				if (item.isFormField()) {
				    String name = item.getFieldName();
				    String value = item.getString();
				    if ("reportId".equals(name))
				    {
				    	reportId = Long.valueOf(value);
				    }
				}
				if (!item.isFormField() && "reportFile".equals(item.getFieldName()))
				{
					reportUploadItem = item;
				}
			}
			
			if (reportUploadItem == null)
			{
				response.getOutputStream().println("{\"success\":false,\"errors\":[{\"Problem during file upload\"}]}");
				return;
			}
			
			byte[] reportContent = reportUploadItem.get();
			
			Report report = getReportService().findReportById(reportId);
			report.setReportContent(reportContent);
			// Workaround for @PreUpdate that do not work.
			new ReportEntityListener().zipReportContent(report);
			getReportService().updateReport(report);
			
//			byte[] data = getReportService().findReportDescriptionById(reportId).getReportContent();
//			FileOutputStream fos = new FileOutputStream("d:/tmp/upload.rptdesign");
//			org.apache.commons.io.IOUtils.write(reportContent, fos);
//			fos.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		response.setContentType("text/xml");
		PrintWriter pw = response.getWriter();
//		response.getOutputStream().println("{success:true}");
		pw.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		pw.println("<response success=\"true\">");
		pw.println("<data>");
		pw.println("<responseMsg>this is a message</responseMsg>");
		pw.println("</data>");
		pw.println("</response>");

		pw.flush();
	}
	
	protected ReportService getReportService() {
		ApplicationContext ctx = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
		ReportService reportService = (ReportService) ctx.getBean("reportService");
		return reportService;
	}
}
